# Licensed under the Apache License, Version 2.0 (the "License"); you may

# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource
from openstack import utils
from openstack import exceptions


class Image(resource.Resource):
    resources_key = 'images'
    resource_key = 'image'
    base_path = '/cloudimages'

    allow_create = True
    allow_list = True
    allow_commit = True

    #: Method for creating a resource (POST, PUT)
    create_method = "POST"
    commit_method = "PATCH"

    _query_mapping = resource.QueryParameters('limit', 'id',
                                              'name', 'status',
                                              '__isregistered',
                                              '__imagetype',
                                              '__whole_image',
                                              '__system__cmkid',
                                              'protected',
                                              'visibility',
                                              'owner',
                                              'container_format',
                                              'disk_format',
                                              'min_disk',
                                              'min_ram',
                                              '__os_bit',
                                              '__platform',
                                              'marker',
                                              'sort_key',
                                              'sort_dir',
                                              '__os_type',
                                              'tag',
                                              'member_status',
                                              '__support_kvm',
                                              '__support_xen',
                                              '__support_largememory',
                                              '__support_diskintensive',
                                              '__support_highperformance',
                                              '__support_xen_gpu_type',
                                              '__support_kvm_gpu_type',
                                              '__support_xen_hana',
                                              '__support_kvm_infiniband',
                                              'virtual_env_type',
                                              'enterprise_project_id')

    name = resource.Body('name')
    description = resource.Body('__description')
    os_type = resource.Body('os_type')
    os_version = resource.Body('__os_version')
    image_url = resource.Body('image_url')
    instance_id = resource.Body('instance_id')
    min_disk = resource.Body('min_disk', type=int)
    is_config = resource.Body('is_config', type=bool)
    cmk_id = resource.Body('cmk_id')
    type = resource.Body('type')
    max_ram = resource.Body('max_ram', type=int)
    min_ram = resource.Body('min_ram', type=int)
    is_quick_import = resource.Body('is_quick_import', type=bool)
    tags = resource.Body('tags', type=list)
    image_tags = resource.Body('image_tags', type=list)
    data_images = resource.Body('data_images', type=list)
    job_id = resource.Body('job_id')
    status = resource.Body('status')
    visibility = resource.Body('visibility')
    created_at = resource.Body('created_at')
    updated_at = resource.Body('updated_at')
    container_format = resource.Body('container_format')
    disk_format = resource.Body('disk_format')
    member_status = resource.Body('member_status')
    virtual_env_type = resource.Body('virtual_env_type')
    enterprise_project_id = resource.Body('enterprise_project_id')
    protected = resource.Body('protected', type=bool)
    virtual_size = resource.Body('virtual_size')
    hw_firmware_type = resource.Body('hw_firmware_type')
    disk_format = resource.Body('disk_format')
    container_format = resource.Body('container_format')
    hw_vif_multiqueue_enabled = resource.Body('hw_vif_multiqueue_enabled')
    checksum = resource.Body('checksum')
    size = resource.Body('size')
    file = resource.Body('file')
    os_bit = resource.Body('__os_bit')
    platform = resource.Body('__platform')
    is_registered = resource.Body('__is_registered')
    os_type = resource.Body('__os_type')
    image_source_type = resource.Body('__image_source_type')
    imagetype = resource.Body('__imagetype')
    originalimagename = resource.Body('__originalimagename')
    backup_id = resource.Body('__backup_id')
    productcode = resource.Body('__productcode')
    image_size = resource.Body('__image_size')
    support_fc_inject = resource.Body('__support_fc_inject')
    data_origin = resource.Body('__data_origin')

    def create(self, session, prepend_key=False, base_path=None):
        # Override here to override prepend_key default value
        return super(Image, self).create(session, prepend_key, base_path)

    def _action(self, session, request_body, image_id):
        url = utils.urljoin(self.base_path, image_id)
        response = session.patch(url, json=request_body)
        exceptions.raise_from_response(response)
        return response

    def update_image_details(self, session, image_id, command_list):
        request_body = command_list
        response = self._action(
            session=session,
            request_body=request_body,
            image_id=image_id)
        return self._to_object(session, response)

    def _to_object(self, session, response):
        has_body = (
            self.has_body
            if self.create_returns_body is None
            else self.create_returns_body
        )
        microversion = self._get_microversion(session, action='create')
        self.microversion = microversion
        self._translate_response(response, has_body=has_body)
        if self.has_body and self.create_returns_body is False:
            return self.fetch(session)
        return self



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from otcextensions.osclient.vlb.v3 import listener
from otcextensions.tests.unit.osclient.vlb.v3 import fakes


class TestListener(fakes.TestVLB):

    def setUp(self):
        super(TestListener, self).setUp()

    def test_flatten(self):
        obj = fakes.FakeListener.create_one()

        flat_data = listener._flatten_listener(obj)

        data = (
            flat_data['client_ca_tls_container_ref'],
            flat_data['connection_limit'],
            flat_data['created_at'],
            flat_data['default_pool_id'],
            flat_data['default_tls_container_ref'],
            flat_data['description'],
            flat_data['http2_enable'],
            flat_data['id'],
            flat_data['insert_headers'],
            flat_data['is_admin_state_up'],
            flat_data['name'],
            flat_data['project_id'],
            flat_data['security_policy_id'],
            flat_data['sni_match_algo'],
            flat_data['protocol'],
            flat_data['protocol_port'],
            flat_data['sni_container_refs'],
            flat_data['updated_at'],
            flat_data['tls_ciphers_policy'],
            flat_data['enable_member_retry'],
            flat_data['keepalive_timeout'],
            flat_data['client_timeout'],
            flat_data['member_timeout'],
            flat_data['ipgroup'],
            flat_data['transparent_client_ip_enable'],
            flat_data['enhance_l7policy_enable']
        )

        cmp_data = (
            obj.client_ca_tls_container_ref,
            obj.connection_limit,
            obj.created_at,
            obj.default_pool_id,
            obj.default_tls_container_ref,
            obj.description,
            obj.http2_enable,
            obj.id,
            obj.insert_headers,
            obj.is_admin_state_up,
            obj.name,
            obj.project_id,
            obj.security_policy_id,
            obj.sni_match_algo,
            obj.protocol,
            obj.protocol_port,
            obj.sni_container_refs,
            obj.updated_at,
            obj.tls_ciphers_policy,
            obj.enable_member_retry,
            obj.keepalive_timeout,
            obj.client_timeout,
            obj.member_timeout,
            obj.ipgroup,
            obj.transparent_client_ip_enable,
            obj.enhance_l7policy

        )

        self.assertEqual(data, cmp_data)

    def test_add_tags_to_listener_output(self):
        obj = fakes.FakeListener.create_one()

        column = ()
        data = ()
        verify_column = (
            'tags',
        )
        verify_data = (
            ('value=val-tags, key=key-tags',)
        )

        data, column = listener._add_tags_to_listener_obj(
            obj, data, column)

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)

    def test_normalize_tags(self):
        tags = [
            'key1=value',
            'key2=',
            'key3'
        ]

        verify_result = [
            {'key': 'key1', 'value': 'value'},
            {'key': 'key2', 'value': ''},
            {'key': 'key3', 'value': ''}
        ]

        result = listener._normalize_tags(tags)

        self.assertEqual(result, verify_result)

    def test_add_loadbalancer_to_listener_output(self):
        obj = fakes.FakeListener.create_one()

        column = ()
        data = ()
        verify_column = (
            'loadbalancer_id_1',
            'loadbalancer_id_2',
        )
        verify_data = (
            'loadbalancer-id-1',
            'loadbalancer-id-2',
        )

        data, column = listener._add_loadbalancers_to_listener_obj(
            obj, data, column)

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Flavor v3 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)

DB_TYPE_CHOICES = ['mysql', 'postgresql', 'sqlserver']


class ListDatabaseFlavors(command.Lister):

    _description = _("List database flavors.")
    columns = ('name', 'instance_mode', 'vcpus', 'ram')

    def get_parser(self, prog_name):
        parser = super(ListDatabaseFlavors, self).get_parser(prog_name)

        parser.add_argument(
            'database',
            metavar='{' + ','.join(DB_TYPE_CHOICES) + '}',
            type=lambda s: s.lower(),
            choices=DB_TYPE_CHOICES,
            help=_('Name of the datastore Engine.'),
        )

        parser.add_argument(
            'version',
            metavar='<version>',
            help=_('Specifies the database version.'),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        data = client.flavors(
            datastore_name=parsed_args.database,
            version_name=parsed_args.version)

        return (
            self.columns,
            (utils.get_item_properties(
                s,
                self.columns,
            ) for s in data)
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk import sdk_resource

# Only a basic tests for extended functionality are implemented since
# the _list code is copied from sdk.resource to override headers
# TODO(agoncharov) make sense to implement (copy) existing base_resource
# tests from SDK

PROJECT_ID = '123'
IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'id': IDENTIFIER,
    'links': '1',
    'name': '2',
    'ram': 3,
}


class Res(sdk_resource.Resource):

    base_path = '/'
    allow_list = True


class TestBaseResource(base.TestCase):

    def setUp(self):
        super(TestBaseResource, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get_project_id = mock.Mock(return_value=PROJECT_ID)

        self.sot = Res(**EXAMPLE)

        # inject some properties to enable methods
        # self.sot.allow_list = True
        # self.sot.base_path = '/'

        self.base_path = self.sot.base_path

        self.headers = {"Content-Type": "application/json"}

    def test_basic(self):
        sot = sdk_resource.Resource()
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)

    def test_list_defaults(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = []

        self.sess.get.return_value = mock_response

        result = list(self.sot.list(self.sess))

        self.sess.get.assert_called_once_with(
            self.base_path,
            params={},
        )

        self.assertEqual([], result)

    def test_list_override_headers(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = [EXAMPLE]

        self.sess.get.return_value = mock_response

        result = list(self.sot.list(self.sess, headers={'a': 'b'}))

        self.sess.get.assert_called_once_with(
            self.base_path,
            headers={"a": "b"},
            params={},
        )

        self.assertEqual([sdk_resource.Resource(**EXAMPLE)], result)

    def test_list_override_endpoint(self):
        # sot = _base.Resource()

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = [EXAMPLE]

        self.sess.get.return_value = mock_response

        result = list(self.sot.list(
            self.sess,
            headers={'a': 'b'},
            endpoint_override='http:example.com'))

        self.sess.get.assert_called_once_with(
            self.base_path,
            headers={"a": "b"},
            endpoint_override='http:example.com',
            params={},
        )

        self.assertEqual([self.sot], result)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.dws.v1 import tag
from unittest import mock


class TestTag(base.TestCase):
    def setUp(self):
        super(TestTag, self).setUp()
        self.cluster_id = 'example_cluster_id'
        self.proxy = mock.Mock()
        self.mock_response = mock.Mock()
        self.mock_response.status_code = 204

    def test_basic(self):
        sot = tag.Tag()
        self.assertEqual('/clusters/%(cluster_id)s/tags', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        example = {
            'key': 'example_key',
            'value': 'example_value',
            'cluster_id': 'example_cluster_id',
        }
        sot = tag.Tag(**example)
        self.assertEqual(example['key'], sot.key)
        self.assertEqual(example['value'], sot.value)
        self.assertEqual(example['cluster_id'], sot.cluster_id)

    def test_batch_create_cluster_tags(self):
        tags = [
            {'key': 'key1', 'value': 'value1'},
            {'key': 'key2', 'value': 'value2'},
        ]
        tag_instance = tag.Tag()
        self.proxy.post.return_value = self.mock_response
        tag_instance.manage_tags_batch(
            self.proxy, self.cluster_id, tags, 'create'
        )
        self.proxy.post.assert_called_once_with(
            'clusters/example_cluster_id/tags/action',
            json={'action': 'create', 'tags': tags},
        )

    def test_batch_delete_cluster_tags(self):
        tags = [{'key': 'key1'}, {'key': 'key2'}]
        tag_instance = tag.Tag()
        self.proxy.post.return_value = self.mock_response
        tag_instance.manage_tags_batch(
            self.proxy, self.cluster_id, tags, 'delete'
        )
        self.proxy.post.assert_called_once_with(
            'clusters/example_cluster_id/tags/action',
            json={'action': 'delete', 'tags': tags},
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from otcextensions.sdk import sdk_proxy
from otcextensions.sdk.cce.v1 import cluster as _cluster
from otcextensions.sdk.cce.v1 import cluster_node as _cluster_node


class Proxy(sdk_proxy.Proxy):

    skip_discovery = True

    # ======== Cluster ========
    def clusters(self):
        """List all Clusters.

        :returns: a generator of
            (:class:`~otcextensions.sdk.cce.v1.cluster.Cluster`) instances
        """
        return self._list(_cluster.Cluster, paginated=False)

    def get_cluster(self, cluster):
        """Get the cluster by UUID

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.cce.v1.cluster.Cluster`

        :returns: instance of
            :class:`~otcextensions.sdk.cce.v1.cluster.Cluster`
        """
        return self._get(
            _cluster.Cluster, cluster,
        )

    def create_cluster(self, **attrs):
        """Create a cluster from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.cce.v1.cluster.Cluster`,
            comprised of the properties on the Cluster class.
        :returns: The results of cluster creation
        :rtype: :class:`~otcextensions.sdk.cce.v1.cluster.Cluster`
        """
        return self._create(
            _cluster.Cluster, prepend_key=False, **attrs
        )

    def find_cluster(self, name_or_id, ignore_missing=True):
        """Find a single cluster

        :param name_or_id: The name or ID of a cluster
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the group does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent group.

        :returns: ``None``
        """
        return self._find(
            _cluster.Cluster, name_or_id,
            ignore_missing=ignore_missing,
        )

    def delete_cluster(self, cluster, ignore_missing=True):
        """Delete a cluster

        :param cluster: The value can be the ID of a cluster
             or a :class:`~otcextensions.sdk.cce.v1.cluster.Cluster`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the group does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent cluster.
        """
        return self._delete(
            _cluster.Cluster, cluster, ignore_missing=ignore_missing,
        )

    # ======== Cluster Nodes ========
    def cluster_nodes(self, cluster):
        """List all Cluster nodes.

        :param cluster: The value can be the ID of a cluster
             or a :class:`~otcextensions.sdk.cce.v1.cluster.Cluster`
             instance.

        :returns: a generator of
            (:class:`~otcextensions.sdk.cce.v1.cluster_node.ClusterNode`)
            instances
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return self._list(
            _cluster_node.ClusterNode, cluster_uuid=cluster.id,
            paginated=False
        )

    def get_cluster_node(self, cluster, node_id):
        """Get the cluster node by it's UUID

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.cce.v1.cluster.Cluster`
        :param node_id: Cluster node id to be fetched

        :returns: instance of
            :class:`~otcextensions.sdk.cce.v1.cluster_node.ClusterNode`
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return self._get(
            _cluster_node.ClusterNode,
            node_id,
            cluster_uuid=cluster.id,
        )

    def find_cluster_node(self, cluster, node):
        """Find the cluster node by it's UUID or name

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.cce.v1.cluster.Cluster`
        :param node: Cluster node id or name to be fetched

        :returns: instance of
            :class:`~otcextensions.sdk.cce.v1.cluster_node.ClusterNode`
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return self._find(
            _cluster_node.ClusterNode,
            node,
            cluster_uuid=cluster.id,
        )

    def delete_cluster_nodes(self, cluster, node_names):
        """Delete nodes from the cluster

        :param cluster: The value can be the ID of a cluster
             or a :class:`~otcextensions.sdk.cce.v1.cluster.Cluster`
             instance.
        :param node_names: List of node names to be deleted.
            Can be also a single node name.
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.delete_nodes(
            self,
            node_names,
        )

    def add_node(self, cluster, **attrs):
        """Add a new node to the cluster

        :param cluster: The value can be the ID of a cluster
             or a :class:`~otcextensions.sdk.cce.v1.cluster.Cluster`
             instance.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.cce.v1.cluster_node.ClusterHost`,
            comprised of the properties on the ClusterHost class.
        :returns: The results of config creation
        :rtype: :class:`~otcextensions.sdk.cce.v1.config.Config`
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return self._create(
            _cluster_node.ClusterNode,
            cluster_uuid=cluster.id,
            **attrs
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.cce.v1 import cluster_node

OS_HEADERS = {
    'Content-Type': 'application/json',
}

EXAMPLE_LIST = {
    'kind': 'list',
    'apiVersion': 'v1',
    'metadata': {},
    'spec': {
        'hostList': [{
            'kind': 'host',
            'apiVersion': 'v1',
            'metadata': {
                'name': 'ag-test-node-1',
                'uuid': '580add6d-db4a-4571-ae02-9b3364818f8a',
                'spaceuuid': 'de213bc4-4949-2a70-1a28-ae6c54e4ea23',
                'createAt': '2018-04-03 13:37:27.993379777 +0000 UTC',
                'updateAt': '2018-04-03 13:37:27.993379777 +0000 UTC'},
            'spec': {
                'clusteruuid': '5a66a449-668c-492f-8c33-5cdbdeaadd2e',
                'clustername': 'ag-test',
                'flavor': 's1.medium',
                'cpu': 1,
                'memory': 4096,
                'label': 'fake',
                'status': {
                    'capacity': {
                        'cpu': '',
                        'memory': '',
                        'pods': ''
                    },
                    'allocatable': {
                        'cpu': '',
                        'memory': '',
                        'pods': ''
                    },
                    'conditions': None,
                    'addresses': None,
                    'daemonEndpoints': {
                        'kubeletEndpoint': {'Port': 0}
                    },
                    'nodeInfo': {
                        'machineID': '',
                        'systemUUID': '',
                        'bootID': '',
                        'kernelVersion': '',
                        'osImage': '',
                        'containerRuntimeVersion': '',
                        'kubeletVersion': '',
                        'kubeProxyVersion': ''},
                    'images': None
                }
            },
            'replicas': 1,
            'status': 'BUILD'
        }]
    }
}

EXAMPLE_CREATE = {
    "kind": "host",
    "apiVersion": "v1",
    "spec": {
        "flavor": "s1.medium",
        "label": "",
        "volume": [{
            "diskType": "root",
            "diskSize": 40,
            "volumeType": "SAS"
        }, {
            "diskType": "data",
            "diskSize": 100,
            "volumeType": "SATA"
        }],
        "sshkey": "SSHkey-1864",
        "snat": False,
        "az": "eu-de-01",
        "tags": ["aaa.111", "bbb.222"]
    },
    "replicas": 1
}


class TestClusterNode(base.TestCase):

    def setUp(self):
        super(TestClusterNode, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.delete = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.get = mock.Mock()

        self.sot = cluster_node.ClusterNode(cluster_uuid='cluster_uuid')

    def test_basic(self):
        sot = cluster_node.ClusterNode()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual(None, sot.resources_key)
        self.assertEqual('/clusters/%(cluster_uuid)s/hosts',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertTrue(sot.allow_delete)

    # def test_make_it(self):
    #     obj = EXAMPLE_LIST[0]
    #     sot = cluster.Cluster.existing(**obj)
    #     self.assertEqual(obj['metadata']['uuid'], sot.id)
    #     self.assertEqual(obj['kind'], sot.kind)

    def test_list(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = copy.deepcopy(EXAMPLE_LIST)

        self.sess.get.return_value = mock_response

        result = list(
            self.sot.list(
                self.sess,
                headers=OS_HEADERS,
                cluster_uuid='cluster_uuid',
            )
        )

        self.sess.get.assert_called_once_with(
            '/clusters/%s/hosts' % self.sot.cluster_uuid,
            params={},
            headers=OS_HEADERS
        )

        expected_list = [
            cluster_node.ClusterNode.existing(
                **EXAMPLE_LIST['spec']['hostList'][0]),
        ]

        self.assertEqual(expected_list, result)

    def test_get(self):
        host = EXAMPLE_LIST['spec']['hostList'][0]
        sot = cluster_node.ClusterNode.existing(
            id=host['metadata']['uuid'],
            cluster_uuid='cluster_uuid'
        )

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = host.copy()

        self.sess.get.return_value = mock_response

        result = sot.get(self.sess)

        self.sess.get.assert_called_once_with(
            'clusters/%s/hosts/%s' % (
                'cluster_uuid',
                sot.id
            ),
            headers=OS_HEADERS
        )

        self.assertDictEqual(
            cluster_node.ClusterNode.existing(**host).to_dict(),
            result.to_dict()
        )

    def test_delete(self):
        sot = cluster_node.ClusterNode.existing(
            cluster_uuid='bla',
            **EXAMPLE_LIST['spec']['hostList'][0])

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = None

        self.sess.delete.return_value = mock_response

        sot.delete(self.sess)

        self.sess.delete.assert_called_once_with(
            '/clusters/%s/hosts' % 'bla',
            json={'hosts': [{'name': 'ag-test-node-1'}]},
            headers=OS_HEADERS)

    def test_create(self):
        sot = cluster_node.ClusterNode.new(
            cluster_uuid='bla',
            **EXAMPLE_CREATE)

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = [{}]

        self.sess.post.return_value = mock_response

        sot.create(self.sess)

        self.sess.post.assert_called_once_with(
            '/clusters/%s/hosts' % 'bla',
            json=EXAMPLE_CREATE,
            headers=OS_HEADERS)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class BaseUserSpec(resource.Resource):
    id = resource.Body('id')
    name = resource.Body('name')


class UserInfoSpec(resource.Resource):
    id = resource.Body('id')
    name = resource.Body('name')
    domain = resource.Body('domain', type=BaseUserSpec)


class MetaDataSpec(resource.Resource):
    count = resource.Body('count', type=int)
    marker = resource.Body('marker')


class Trace(resource.Resource):
    base_path = '/traces'
    resources_key = 'traces'

    allow_list = True

    _query_mapping = resource.QueryParameters(
        'trace_type', 'limit', 'from', 'next', 'to', 'tracker_name',
        'service_type', 'user', 'resource_id', 'resource_name',
        'resource_type', 'trace_id', 'trace_name', 'trace_rating')

    resource_id = resource.Body('resource_id')
    trace_name = resource.Body('trace_name')
    trace_rating = resource.Body('trace_rating')
    trace_type = resource.Body('trace_type')
    request = resource.Body('request')
    response = resource.Body('response')
    code = resource.Body('code')
    api_version = resource.Body('api_version')
    message = resource.Body('message')
    record_time = resource.Body('record_time')
    trace_id = resource.Body('trace_id')
    time = resource.Body('time')
    user = resource.Body('user', type=UserInfoSpec)
    service_type = resource.Body('service_type')
    resource_type = resource.Body('resource_type')
    source_ip = resource.Body('source_ip')
    resource_name = resource.Body('resource_name')
    request_id = resource.Body('request_id')
    location_info = resource.Body('location_info')
    endpoint = resource.Body('endpoint')
    resource_url = resource.Body('resource_url')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class ApiAuthResult(resource.Resource):
    status = resource.Body('status')
    error_msg = resource.Body('error_msg')
    error_code = resource.Body('error_code')
    api_name = resource.Body('api_name')
    app_name = resource.Body('app_name')


class ApiAuthInfo(resource.Resource):
    base_path = f'apigw/instances/%(gateway_id)s/app-auths'
    resources_key = 'auths'

    _query_mapping = resource.QueryParameters('limit', 'offset',
                                              'app_id', 'api_id',
                                              'api_name', 'group_id',
                                              'group_name', 'env_id',
                                              'app_name')

    allow_list = True
    allow_create = True
    gateway_id = resource.URI('gateway_id')
    env_id = resource.Body('env_id')
    app_ids = resource.Body('app_ids', type=list)
    api_ids = resource.Body('api_ids', type=list)

    api_id = resource.Body('api_id')
    api_name = resource.Body('api_name')
    group_name = resource.Body('group_name')
    api_type = resource.Body('api_type', type=int)
    api_remark = resource.Body('api_remark')
    app_name = resource.Body('app_name')
    app_remark = resource.Body('app_remark')
    app_type = resource.Body('app_type')
    app_creator = resource.Body('app_creator')
    publish_id = resource.Body('publish_id')
    group_id = resource.Body('group_id')
    auth_result = resource.Body('auth_result', type=ApiAuthResult)
    auth_time = resource.Body('auth_time')
    id = resource.Body('id')
    app_id = resource.Body('app_id')
    auth_role = resource.Body('auth_role')
    auth_tunnel = resource.Body('auth_tunnel')
    auth_whitelist = resource.Body('auth_whitelist', type=list)
    auth_blacklist = resource.Body('auth_blacklist', type=list)
    visit_params = resource.Body('visit_params')
    visit_param = resource.Body('visit_param')
    roma_app_type = resource.Body('roma_app_type')
    env_name = resource.Body('env_name')
    run_env_name = resource.Body('run_env_name')

    def _authorize_apps(self, session, gateway_id, **attrs):
        uri = f'apigw/instances/{gateway_id}/app-auths'
        response = session.post(uri, json=attrs)
        exceptions.raise_from_response(response)
        data = response.json()
        resources = data[self.resources_key]
        for raw_resource in resources:
            value = self.existing(
                connection=session._get_connection(),
                **raw_resource)
            yield value

    def _cancel_auth(self, session, gateway_id, app_auth_id):
        uri = f'apigw/instances/{gateway_id}/app-auths/{app_auth_id}'
        response = session.delete(uri)
        exceptions.raise_from_response(response)
        return None


class ApiAuth(ApiAuthInfo):
    allow_create = False
    resources_key = 'apis'



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base

from otcextensions.sdk.rds.v3 import flavor

EXAMPLE = {
    "vcpus": "1",
    "ram": 2,
    "spec_code": "rds.mysql.c2.medium.ha",
    "instance_mode": "ha",
    "group_type": "normal",
    "version_name": [
        "8.0"
    ],
    "az_status": {
        "eu-de-02": "normal",
        "eu-de-01": "normal",
        "eu-de-03": "normal"
    },
    "az_desc": {
        "eu-de-02": "eu-de-02",
        "eu-de-01": "eu-de-01",
        "eu-de-03": "eu-de-03"
    }
}


class TestFlavor(base.TestCase):

    def test_basic(self):
        sot = flavor.Flavor()

        self.assertEqual('/flavors/%(datastore_name)s', sot.base_path)
        self.assertEqual('flavors', sot.resources_key)
        self.assertIsNone(sot.resource_key)

        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)
        self.assertDictEqual({'limit': 'limit',
                              'marker': 'marker',
                              'version_name': 'version_name',
                              'spec_code': 'spec_code'},
                             sot._query_mapping._mapping)

    def test_make_it(self):

        sot = flavor.Flavor(**EXAMPLE)
        self.assertEqual(EXAMPLE['spec_code'], sot.spec_code)
        self.assertEqual(EXAMPLE['spec_code'], sot.name)
        self.assertEqual(EXAMPLE['vcpus'], sot.vcpus)
        self.assertEqual(EXAMPLE['ram'], sot.ram)
        self.assertEqual(EXAMPLE['instance_mode'], sot.instance_mode)
        self.assertEqual(EXAMPLE['group_type'], sot.group_type)
        self.assertEqual(EXAMPLE['version_name'], sot.version_name)
        self.assertEqual(EXAMPLE['az_status'], sot.az_status)
        self.assertEqual(EXAMPLE['az_desc'], sot.az_desc)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk.dms.v1 import _base


class MaintenanceWindow(_base.Resource):
    """DMS Maintenance window resource"""
    resources_key = 'maintain_windows'
    base_path = '/instances/maintain-windows'

    # capabilities
    allow_list = True

    #: Properties
    #: Indicates the sequential number of a maintenance time window.
    seq = resource.Body('seq', type=int)
    #: Indicates the time at which a maintenance time window starts.
    begin = resource.Body('begin')
    #: Indicates the time at which a maintenance time window ends.
    end = resource.Body('end')
    #: Indicates whether a maintenance time window is set to the
    #: default time segment.
    is_default = resource.Body('default', type=bool)

    @classmethod
    def list(cls, session, **params):
        return super(MaintenanceWindow, cls).list_override(session, **params)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from openstack import _log

from otcextensions.sdk.dms.v1 import _base

_logger = _log.setup_logging('openstack')


class Quota(_base.Resource):
    """DMS Quota resource"""
    resources_key = 'quotas.resources'
    base_path = '/quotas/dms'

    # capabilities
    allow_list = True

    #: Properties
    #: Quota of type
    type = resource.Body('type')
    #: Quota amount has been used
    used = resource.Body('used', type=int)
    #: Quota max amount
    max = resource.Body('max', type=int)
    #: Quota min amount
    min = resource.Body('min', type=int)
    #: Quota amount
    quota = resource.Body('quota', type=int)

    @classmethod
    def list(cls, session, paginated=False,
             endpoint_override=None, headers=None, requests_auth=None,
             **params):
        return super(Quota, cls).list_ext(
            session, paginated,
            endpoint_override, headers,
            **params)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import api_call

EXAMPLE_STAT = {
    'max_latency': 500,
    'avg_latency': 123.45,
    'req_count': 1000,
    'req_count_2xx': 800,
    'req_count_4xx': 150,
    'req_count_5xx': 50,
    'req_count_error': 200,
    'max_inner_latency': 300,
    'avg_inner_latency': 120.5,
    'max_backend_latency': 450,
    'avg_backend_latency': 200.0,
    'output_throughput': 1.23,
    'input_throughput': 0.89,
    'current_minute': 5,
    'cycle': '5m',
    'api_id': 'api-123',
    'group_id': 'group-456',
    'provider': 'custom',
    'req_time': '2025-01-01T12:00:00Z',
    'register_time': '2025-01-01T11:00:00Z',
    'status': 1
}

EXAMPLE_RESULT = {
    'code': '200',
    'msg': 'OK',
    'start_time': '2025-01-01T12:00:00Z',
    'end_time': '2025-01-01T12:05:00Z',
    'list': [EXAMPLE_STAT]
}


class TestApiCallResult(base.TestCase):

    def test_basic(self):
        sot = api_call.ApiCallResult()
        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):
        sot = api_call.ApiCallResult(**EXAMPLE_RESULT)
        self.assertEqual('200', sot.code)
        self.assertEqual('OK', sot.msg)
        self.assertEqual('2025-01-01T12:00:00Z', sot.start_time)
        self.assertEqual(1, len(sot.api_list))
        stat = sot.api_list[0]
        self.assertEqual(500, stat.max_latency)
        self.assertEqual('api-123', stat.api_id)
        self.assertEqual('group-456', stat.group_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class LoadBalancer(resource.Resource):
    resource_key = 'loadbalancer'
    resources_key = 'loadbalancers'
    base_path = '/elb/loadbalancers'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'availability_zone_list', 'id', 'description',
        'name', 'publicips', 'provisioning_status',
        'operating_status', 'guaranteed',
        'vip_address', 'ip_version',
        'deletion_protection_enable', 'subnet_type',
        'vip_port_id', 'vip_subnet_cidr_id',
        'l4_flavor_id', 'l7_flavor_id', 'member_device_id',
        is_admin_state_up='admin_state_up',
        availability_zones='availability_zone_list',
        floating_ips='publicips', is_guaranteed='guaranteed',
        ip_address='vip_address', port_id='vip_port_id',
        subnet_id='vip_subnet_cidr_id'
    )

    # Properties
    #: Name of the target Octavia availability zone
    availability_zones = resource.Body('availability_zone_list', type=list)
    #: Timestamp when the load balancer was created
    created_at = resource.Body('created_at')
    #: The load balancer description
    description = resource.Body('description')
    #: The load balancer description
    deletion_protection_enable = resource.Body('deletion_protection_enable',
                                               type=bool)
    #: EIP bound to the load balancer.
    eips = resource.Body('eips', type=list)
    #: FIP
    floating_ip = resource.Body('publicip', type=list, list_type=dict)
    #: Assigned FIPs
    floating_ips = resource.Body('publicips', type=list, list_type=dict)
    #: Specifies whether the load balancer is a dedicated load balancer.
    is_guaranteed = resource.Body('guaranteed')
    #: The administrative state of the load balancer *Type: bool*
    is_admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Specifies whether to enable cross-VPC backend.
    ip_target_enable = resource.Body('ip_target_enable')
    #: The Layer-4 flavor.
    l4_flavor_id = resource.Body('l4_flavor_id')
    #: Specifies the Layer-7 flavor.
    l7_flavor_id = resource.Body('l7_flavor_id')
    #: List of listeners associated with this load balancer
    listeners = resource.Body('listeners', type=list, list_type=dict)
    #: Load balancer name.
    name = resource.Body('name')
    #: Network id
    network_ids = resource.Body('elb_virsubnet_ids', type=list)
    #: Subnet type
    subnet_type = resource.Body('elb_virsubnet_type')
    #: Operating status of the load balancer
    operating_status = resource.Body('operating_status')
    #: List of pools associated with this load balancer
    pools = resource.Body('pools', type=list, list_type=dict)
    #: The ID of the project this load balancer is associated with.
    project_id = resource.Body('project_id')
    #: Provider name for the load balancer.
    provider = resource.Body('provider')
    #: The provisioning status of this load balancer
    provisioning_status = resource.Body('provisioning_status')
    #: Tags added to the load balancer
    tags = resource.Body('tags', type=list)
    #: Timestamp when the load balancer was last updated
    updated_at = resource.Body('updated_at')
    #: Specifies the private IP address of the load balancer.
    ip_address = resource.Body('vip_address')
    #: Specifies the ID of the port bound to the private IPv4 address
    # of the load balancer.
    port_id = resource.Body('vip_port_id')
    #: VIP subnet ID
    subnet_id = resource.Body('vip_subnet_cidr_id')
    #: Router id
    vpc_id = resource.Body('vpc_id')



#   Copyright 2013 Nebula Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import random
import uuid

import mock

from otcextensions.sdk.volume_backup.v2 import backup_policy

from otcextensions.tests.unit.osclient import test_base


class TestVolumeBackup(test_base.TestCommand):

    def setUp(self):
        super(TestVolumeBackup, self).setUp()

        self.app.client_manager.volume_backup = mock.Mock()
        self.client = self.app.client_manager.volume_backup


class FakePolicy(test_base.Fake):
    """Fake one or more VBS Policy"""

    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'name-' + uuid.uuid4().hex,
            'policy_resource_count': random.randint(0, 20),
            'tags': [{'key': 't1', 'value': uuid.uuid4().hex}],
            'scheduled_policy': {
                'remain_first_backup_of_curMonth': 'Y',
                'rentention_num': random.randint(0, 100),
                'frequency': random.randint(0, 100),
                'start_time': 'st-' + uuid.uuid4().hex,
                'status': 'ON'
            },
        }
        obj = backup_policy.BackupPolicy.existing(**object_info)
        return obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.dms.v1 import queue

EXAMPLE = {
    "id": "9bf46390-38a2-462d-b392-4d5b2d519c55",
    "name": "queue_001",
    "description": "test1",
    "redrive_policy": "enable",
    "max_consume_count": 1,
    "retention_hours": 7
}


class TestQueue(base.TestCase):

    example = EXAMPLE
    objcls = queue.Queue

    def test_basic(self):
        sot = self.objcls()

        self.assertEqual('/queues', sot.base_path)

        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

        self.assertDictEqual({
            'include_deadletter': 'include_deadletter',
            'include_messages_num': 'include_messages_num',
            'limit': 'limit',
            'marker': 'marker'},
            sot._query_mapping._mapping
        )

    def test_make_it(self):

        sot = self.objcls(**self.example)
        self.assertEqual(self.example['id'], sot.id)
        self.assertEqual(self.example['name'], sot.name)
        self.assertEqual(self.example['description'], sot.description)
        self.assertEqual(self.example['max_consume_count'],
                         sot.max_consume_count)
        self.assertEqual(self.example['redrive_policy'],
                         sot.redrive_policy)
        self.assertEqual(self.example['retention_hours'], sot.retention_hours)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Update API in gateway
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')
attrs = {
    "group_id": "group_id",
    "name": "test_api_001",
    "auth_type": "IAM",
    "backend_type": "HTTP",
    "req_protocol": "HTTP",
    "req_uri": "/test/http",
    "remark": "Mock backend API",
    "type": 2,
    "req_method": "GET",
    "result_normal_sample": "Example success response",
    "result_failure_sample": "Example failure response",
    "tags": ["httpApi"],
    "backend_api": {
        "req_protocol": "HTTP",
        "req_method": "GET",
        "req_uri": "/test/benchmark",
        "timeout": 5000,
        "retry_count": "-1",
        "url_domain": "192.168.189.156:12346"
    },
}
environment = conn.apig.update_api(
    api="env_id",
    gateway="gateway_id",
    **attrs
)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class AppCode(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/apps/%(app_id)s/app-codes'
    resources_key = 'app_codes'

    gateway_id = resource.URI('gateway_id')
    app_id = resource.URI('app_id')

    app_code = resource.Body('app_code')
    id = resource.Body('id')
    create_time = resource.Body('create_time')
    allow_fetch = True
    allow_delete = True
    allow_create = True
    allow_list = True

    def _generate_app_code(self, session, gateway, app, **attrs):
        url = f'/apigw/instances/{gateway.id}/apps/{app.id}/app-codes'
        response = session.put(url, json=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource

from otcextensions.sdk.kms.v1 import _base


class Key(_base.Resource):
    list_path = '/kms/list-keys'
    get_path = '/kms/describe-key'
    create_path = '/kms/create-key'

    resources_key = 'key_details'
    resource_key = 'key_info'

    allow_list = True
    allow_create = True
    allow_update = False
    allow_fetch = True

    # Properties
    #: Secret key ID
    key_id = resource.Body('key_id', alternate_id=True)
    #: User domain name
    domain_name = resource.Body('domain_name')
    #: User domain id
    domain_id = resource.Body('domain_id')
    #: Secret key alias
    key_alias = resource.Body('key_alias')
    #: Secret area
    realm = resource.Body('realm')
    #: Secret key description
    key_description = resource.Body('key_description')
    #: Secret key creation date
    creation_date = resource.Body('creation_date')
    #: Scheduled deletion time
    scheduled_deletion_date = resource.Body('scheduled_deletion_date')
    #: Key state. (2, enabled; 3, disabled; 4: sheduled deleted)
    key_state = resource.Body('key_state')
    #: Default key flag. (default 1 else 0)
    default_key_flag = resource.Body('default_key_flag')
    #: Secret key type.
    key_type = resource.Body('key_type')
    #: Error code when create a secret key
    error_code = resource.Body('error_code')
    #: Error message when create a secret key
    error_msg = resource.Body('error_msg')

    def fetch(self, session, requires_id=None,
              base_path=None, error_message=None,
              skip_cache=False,
              *,
              resource_response_key=None,
              microversion=None,
              **params,
              ):
        if not self.allow_fetch:
            raise exceptions.MethodNotSupported(self, "fetch")
        url = self.get_path
        body = {
            'key_id': self.id
        }
        session = self._get_session(session)
        response = session.post(url, json=body)
        self._translate_response(response, has_body=True)

        return self

    def enable(self, session):
        session = self._get_session(session)
        return self._action(session, 'enable-key', {'key_id': self.id})

    def disable(self, session):
        session = self._get_session(session)
        return self._action(session, 'disable-key', {'key_id': self.id})

    def schedule_deletion(self, session, pending_days=7):
        session = self._get_session(session)
        body = {
            'key_id': self.id,
            'pending_days': pending_days
        }
        return self._action(session, 'schedule-key-deletion', body)

    def cancel_deletion(self, session):
        session = self._get_session(session)
        return self._action(
            session, 'cancel-key-deletion',
            {'key_id': self.id}
        )

    @classmethod
    def list(cls, session, paginated=True, base_path=None, **kwargs):

        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")

        uri = None
        if not hasattr(cls, 'list_path'):
            uri = cls.base_path
        else:
            uri = cls.list_path

        body = {}
        limit = None
        if 'limit' in kwargs:
            limit = kwargs.pop('limit')
            body['limit'] = limit
        if 'marker' in kwargs:
            body['marker'] = kwargs.pop('marker')
        if 'key_state' in kwargs:
            body['key_state'] = kwargs.pop('key_state')
        if 'sequence' in kwargs:
            body['sequence'] = kwargs.pop('sequence')

        total_yielded = 0
        while uri:

            session = cls._get_session(session)

            response = session.post(uri, json=body, **kwargs)

            data = response.json()

            if cls.resources_key:
                resources = data[cls.resources_key]
            else:
                resources = data

            if not isinstance(resources, list):
                resources = [resources]

            marker = None
            for raw_resource in resources:
                raw_resource.pop("self", None)

                if cls.resource_key and cls.resource_key in raw_resource:
                    raw_resource = raw_resource[cls.resource_key]

                value = cls.existing(**raw_resource)

                marker = value.id
                yield value
                total_yielded += 1

            if resources and paginated:
                uri, next_params = cls._get_next_link(
                    uri, response, data, marker, limit, total_yielded)
                body.update(next_params)
            else:
                return

    @classmethod
    def _get_next_link(cls, uri, response, data, marker, limit, total_yielded):
        # service pagination. Returns query for the next page
        next_link = None
        params = {}
        truncated = data.get('truncated', None)
        if truncated:
            next_marker = data.get('next_marker', None)
            if next_marker:
                next_link = uri
                params['marker'] = next_marker
        else:
            next_link = None
        return next_link, params

    @classmethod
    def _get_one_match(cls, search_filter, results):
        """Given a list of results, return the match"""
        the_result = None
        for maybe_result in results:
            id_value = cls._get_id(maybe_result)
            name_value = maybe_result.name
            alias_value = maybe_result.key_alias

            if (id_value == search_filter) or (name_value == search_filter) \
                    or (alias_value == search_filter):
                # Only allow one resource to be found. If we already
                # found a match, raise an exception to show it.
                if the_result is None:
                    the_result = maybe_result
                else:
                    msg = "More than one %s exists with the name '%s'."
                    msg = (msg % (cls.__name__, search_filter))
                    raise exceptions.DuplicateResource(msg)

        return the_result



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from unittest.mock import call

import mock
from openstackclient.tests.unit import utils as tests_utils
from osc_lib import exceptions

from otcextensions.osclient.modelartsv1.v1 import model
from otcextensions.tests.unit.osclient.modelartsv1.v1 import fakes

_COLUMNS = (
    "ai_project",
    "apis",
    "config",
    "created_at",
    "dependencies",
    "execution_code",
    "id",
    "image_address",
    "input_params",
    "install_type",
    "is_publishable",
    "is_subscribed",
    "is_tunable",
    "model_algorithm",
    "model_id",
    "model_metrics",
    "model_name",
    "model_size",
    "model_source",
    "model_status",
    "model_type",
    "model_version",
    "name",
    "output_params",
    "owner_id",
    "project_id",
    "runtime",
    "source_location",
    "source_type",
    "specification",
    "status",
    "tenant_id",
    "version",
    "workspace_id",
)


class TestListModels(fakes.TestModelartsv1):
    objects = fakes.FakeModel.create_multiple(3)

    column_list_headers = ("Id", "Name", "Version", "Model Size")

    data = []

    for s in objects:
        data.append(
            (
                s.model_id,
                s.name,
                s.version,
                s.model_size,
            )
        )

    def setUp(self):
        super(TestListModels, self).setUp()

        self.cmd = model.ListModels(self.app, None)

        self.client.models = mock.Mock()
        self.client.api_mock = self.client.models

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))


class TestCreateModel(fakes.TestModelartsv1):
    _model = fakes.FakeModel.create_one()
    columns = _COLUMNS
    data = fakes.gen_data(_model, columns, model._formatters)

    def setUp(self):
        super(TestCreateModel, self).setUp()

        self.cmd = model.CreateModel(self.app, None)

        self.client.create_model = mock.Mock(return_value=self._model)

    def test_create(self):
        arglist = [
            "test-model",
            "--model-version",
            "1.0.0",
            "--source-location",
            "https://models.obs.xxxx.com/mnist",
            "--model-type",
            "TensorFlow",
        ]
        verifylist = [
            ("name", "test-model"),
            ("model_version", "1.0.0"),
            ("source_location", "https://models.obs.xxxx.com/mnist"),
            ("model_type", "TensorFlow"),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {
            "model_name": "test-model",
            "model_version": "1.0.0",
            "source_location": "https://models.obs.xxxx.com/mnist",
            "model_type": "TensorFlow",
        }
        self.client.create_model.assert_called_with(**attrs)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestShowModel(fakes.TestModelartsv1):
    _model = fakes.FakeModel.create_one()
    columns = _COLUMNS
    data = fakes.gen_data(_model, columns, model._formatters)

    def setUp(self):
        super(TestShowModel, self).setUp()

        self.cmd = model.ShowModel(self.app, None)

        self.client.find_model = mock.Mock(return_value=self._model)
        # self.client.get_model = mock.Mock(return_value=self._model)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            verifylist,
        )

    def test_show(self):
        arglist = [
            self._model.id,
        ]

        verifylist = [("model", self._model.id)]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_model.assert_called_with(
            self._model.id, ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = ["unexist_ma_model"]

        verifylist = [("model", "unexist_ma_model")]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = exceptions.CommandError("Resource Not Found")
        self.client.find_model = mock.Mock(side_effect=find_mock_result)

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual("Resource Not Found", str(e))
        self.client.find_model.assert_called_with(
            "unexist_ma_model", ignore_missing=False
        )


class TestDeleteModel(fakes.TestModelartsv1):
    _model = fakes.FakeModel.create_multiple(2)

    def setUp(self):
        super(TestDeleteModel, self).setUp()

        self.client.find_model = mock.Mock(return_value=self._model[0])
        self.client.delete_model = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = model.DeleteModel(self.app, None)

    def test_delete(self):
        arglist = [
            self._model[0].name,
        ]

        verifylist = [
            ("model", arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_model.assert_called_with(
            self._model[0].name, ignore_missing=False
        )
        self.client.delete_model.assert_called_with(self._model[0].id)
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = []

        for ma_model in self._model:
            arglist.append(ma_model.name)

        verifylist = [("model", arglist)]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_results = self._model
        self.client.find_model = mock.Mock(side_effect=find_mock_results)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        find_calls = []
        delete_calls = []
        for ma_model in self._model:
            find_calls.append(call(ma_model.name, ignore_missing=False))
            delete_calls.append(call(ma_model.id))
        self.client.find_model.assert_has_calls(find_calls)
        self.client.delete_model.assert_has_calls(delete_calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            self._model[0].id,
            "unexist_ma_model",
        ]
        verifylist = [("model", arglist)]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_results = [self._model[0], exceptions.CommandError]
        self.client.find_model = mock.Mock(side_effect=find_mock_results)

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual("1 of 2 Model(s) failed to delete.", str(e))

        self.client.delete_model.assert_any_call(self._model[0].id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
# import six
from openstack import exceptions
from openstack import resource
from openstack import utils


class ProcessingSchemaSpec(resource.Resource):
    # Properties
    #: Attribute name of the source data timestamp.
    timestamp_name = resource.Body('timestamp_name')
    #: Type of the source data timestamp.
    timestamp_type = resource.Body('timestamp_type')
    #: OBS directory generated based on the timestamp format. This parameter
    #:  is mandatory when the timestamp type of the source data is String.
    #: \nValues:
    #: \nyyyy/MM/dd HH:mm:ss
    #: \nMM/dd/yyyy HH:mm:ss
    #: \ndd/MM/yyyy HH:mm:ss
    #: \nyyyy-MM-dd HH:mm:ss
    #: \nMM-dd-yyyy HH:mm:ss
    #: \ndd-MM-yyyy HH:mm:ss
    timestamp_format = resource.Body('timestamp_format')


class ObsDestinationSpec(resource.Resource):

    # Properties
    #: Name of the agency created on IAM. DIS uses an agency to
    #:  access your specified resources.
    agency_name = resource.Body('agency_name')
    #: Offset. Types:
    #: \nLATEST: Maximum offset, indicating that the latest data will
    #:  be extracted. \nTRIM_HORIZON: Minimum offset, indicating
    #:  that the earliest data will be extracted.
    consumer_strategy = resource.Body('consumer_strategy')
    #: User-defined interval at which data is imported from the
    #:  current DIS stream into OBS.
    deliver_time_interval = resource.Body('deliver_time_interval', type=int)
    #: Dump file format. Possible values:
    #: \nText (default)
    destination_file_type = resource.Body('destination_file_type')
    #: Directory to store files that will be dumped to OBS. Different directory
    #:  levels are separated by slashes (/) and cannot start with slashes.
    file_prefix = resource.Body('file_prefix')
    #: Name of the OBS bucket used to store data from the DIS stream.
    obs_bucket_path = resource.Body('obs_bucket_path')
    #: Directory structure of the object file written into OBS. The directory
    #:  structure is in the format of yyyy/MM/dd/HH/mm (time at which the
    #:  dump task was created).
    partition_format = resource.Body('partition_format')
    #: Dump time directory generated based on the timestamp of the
    #:  source data and the configured partition_format.
    processing_schema = resource.Body(
        'processing_schema', type=ProcessingSchemaSpec
    )
    #: Delimiter for the dump file, which is used to separate the
    #:  user data that is written into the dump file.
    record_delimiter = resource.Body('record_delimiter')
    #: Retry Duration
    retry_duration = resource.Body('retry_duration', type=int)
    #: Name of the dump task.
    task_name = resource.Body('task_name')


class DumpTask(resource.Resource):
    base_path = '/streams/%(uri_stream_name)s/transfer-tasks'

    resources_key = 'tasks'

    allow_create = True
    allow_list = True
    allow_commit = True
    allow_delete = True
    allow_fetch = True
    allow_patch = True

    #: Name of the stream to which the dump task belongs.
    uri_stream_name = resource.URI('uri_stream_name')

    # Properties
    #: Time when the dump task is created.
    created_at = resource.Body('create_time')
    #: Dump destination. Possible values:
    #: \nOBS: Data is dumped to OBS.
    destination_type = resource.Body('destination_type')
    #: Exception Strategy
    exception_strategy = resource.Body('exception_strategy')
    #: Latest dump time of the dump task.
    last_transfer_timestamp = resource.Body('last_transfer_timestamp')
    #: Parameter list of OBS to which data in the DIS stream will be dumped.
    obs_destination_descriptor = resource.Body('obs_destination_descriptor',
                                               type=ObsDestinationSpec)
    #: Parameter list of OBS to which data in the DIS stream will be dumped.
    obs_destination_description = resource.Body('obs_destination_description',
                                                type=ObsDestinationSpec)
    #: List of partition dump details.
    partitions = resource.Body('partitions', type=list, list_type=dict)
    #: Dump task status. Possible values:
    #: \nERROR: An error occurs.
    #: \nSTARTING: The dump task is being started.
    #: \nPAUSED: The dump task has been stopped.
    #: \nRUNNING: The dump task is running.
    #: \nDELETE: The dump task has been deleted.
    #: \nABNORMAL: The dump task is abnormal.
    status = resource.Body('state')
    #: ID of the stream to which the dump task belongs.
    stream_id = resource.Body('streamId')
    #: Name of the stream to which the dump task belongs.
    stream_name = resource.Body('stream_name')
    #: ID of the dump task.
    task_id = resource.Body('task_id', alternate_id=True)
    #: Name of the dump task.
    task_name = resource.Body('task_name')

    def _action(self, session, stream_name, action, *task_id):
        """Perform start or stop action on the give dump tasks.
        """
        uri = utils.urljoin('streams', stream_name, 'transfer-tasks/action')
        tasks = [{'id': i} for i in list(*task_id)]
        body = {
            'action': action,
            'tasks': tasks
        }
        response = session.post(uri, json=body)
        exceptions.raise_from_response(response)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource


class TargetService(resource.Resource):
    base_path = '/vpc-endpoint-services/describe'

    _query_mapping = resource.QueryParameters(
        'id',
        'name',
        name='endpoint_service_name',
    )

    # capabilities
    allow_create = False
    allow_fetch = True
    allow_commit = False
    allow_delete = False
    allow_list = True

    requires_id = False

    # Properties
    #: ID of the VPC endpoint service.
    id = resource.Body('id')
    #: Name of the VPC endpoint service.
    service_name = resource.Body('service_name')
    #: Type of the VPC endpoint service.
    service_type = resource.Body('service_type')
    #: Creation time of the VPC endpoint service.
    created_at = resource.Body('created_at')
    #: If the usage of the VPC endpoint service will be charged.
    is_charge = resource.Body('is_charge')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import importlib
import os
import time

import openstack
from openstack import _log
from openstack import utils

from otcextensions.common import exc
from otcextensions.sdk import proxy
from otcextensions.sdk.cloud import cce as _cce
from otcextensions.sdk.cloud import dds as _dds
from otcextensions.sdk.cloud import rds as _rds
from otcextensions.sdk.compute.v2 import server
from otcextensions.sdk.network.v2 import service_provider


_logger = _log.setup_logging('openstack')

__all__ = [
    'register_otc_extensions',
]

_DOC_TEMPLATE = (
    ":class:`{class_name}` for {service_type} aka project")
_PROXY_TEMPLATE = """Proxy for {service_type} aka project

This proxy object could be an instance of
{class_doc_strings}
depending on client configuration and which version of the service is
found on remotely on the cloud.
"""


# List OTC services here
#   Generally it is possible to iterate over known endpoints, but some
#   services requires injection of AK/SK
OTC_SERVICES = {
    'anti_ddos': {
        'service_type': 'anti_ddos',
        'append_project_id': True,
        'endpoint_service_type': 'antiddos',
    },
    'apig': {
        'service_type': 'apig',
        'append_project_id': True,
        'endpoint_service_type': 'apigv2'
    },
    'aomv1': {
        'service_type': 'aomv1',
        'endpoint_service_type': 'aomv1'
    },
    'aomv2': {
        'service_type': 'aomv2',
        'endpoint_service_type': 'aomv2'
    },
    'auto_scaling': {
        'service_type': 'auto_scaling',
        'endpoint_service_type': 'as',
        'append_project_id': True,
    },
    'bms': {
        'service_type': 'bms'
    },
    'cbr': {
        'service_type': 'cbr',
        'append_project_id': True,
    },
    'cce': {
        'service_type': 'cce',
        'endpoint_service_type': 'ccev2.0',
    },
    'ces': {
        'service_type': 'ces',
        'append_project_id': True,
    },
    'cfwv1': {
        'service_type': 'cfwv1',
        'append_project_id': True,
        'endpoint_service_type': 'cfwv1'
    },
    'cfwv2': {
        'service_type': 'cfwv2',
        'append_project_id': True,
        'endpoint_service_type': 'cfwv2'
    },
    'cfwv3': {
        'service_type': 'cfwv3',
        'append_project_id': True,
        'endpoint_service_type': 'cfwv3'
    },
    'cts': {
        'service_type': 'cts',
        'endpoint_service_type': 'cts'
    },
    'ctsv2': {
        'service_type': 'ctsv2',
        'endpoint_service_type': 'ctsv2'
    },
    'ctsv3': {
        'service_type': 'ctsv3',
        'endpoint_service_type': 'ctsv3'
    },
    'css': {
        'service_type': 'css',
    },
    'csbs': {
        'service_type': 'csbs',
        'endpoint_service_type': 'data-protect'
    },
    'dataarts': {
        'service_type': 'dataarts',
    },
    'dcaas': {
        'service_type': 'dcaas',
        'append_project_id': False,
    },
    'dcs': {
        'service_type': 'dcs',
        'append_project_id': True,
    },
    'dds': {
        'service_type': 'dds',
        'endpoint_service_type': 'ddsv3',
        'append_project_id': True
    },
    'deh': {
        'service_type': 'deh',
        'append_project_id': True,
    },
    'ddmv1': {
        'service_type': 'ddmv1',
        'endpoint_service_type': 'ddmv1',
        'append_project_id': True
    },
    'ddmv2': {
        'service_type': 'ddmv2',
        'endpoint_service_type': 'ddmv2',
        'append_project_id': True
    },
    'ddmv3': {
        'service_type': 'ddmv3',
        'endpoint_service_type': 'ddmv3',
        'append_project_id': True
    },
    'dis': {
        'service_type': 'dis',
        'endpoint_service_type': 'disv2'
    },
    'dms': {
        'service_type': 'dms',
        'endpoint_service_type': 'dms',
        'append_project_id': True,
    },
    'dmsv2': {
        'service_type': 'dmsv2',
        'endpoint_service_type': 'dmsv2',
    },
    'dns': {
        'service_type': 'dns',
        'replace_system': True,
    },
    'dli': {
        'service_type': 'dli',
    },
    'drs': {
        'service_type': 'drs',
    },
    'dws': {
        'service_type': 'dws',
        'endpoint_service_type': 'dwsv1'
    },
    'ecs': {
        'service_type': 'ecs',
    },
    'elb': {
        'service_type': 'elb',
        # 'replace_system': True
    },
    'enterprise_dashboard': {
        'service_type': 'enterprise-dashboard',
        'endpoint_service_type': 'enterprise-dashboard'
    },
    'enterprise_dashboard_v1': {
        'service_type': 'enterprise-dashboard-v1',
        'endpoint_service_type': 'enterprise-dashboard-v1'
    },
    'er': {
        'service_type': 'er',
        'append_project_id': True,
        'endpoint_service_type': 'erv3'
    },
    'evpn': {
        'service_type': 'evpn',
        'append_project_id': True,
        'endpoint_service_type': 'evpn'
    },
    'function_graph': {
        'service_type': 'function_graph',
        'append_project_id': True,
        'endpoint_service_type': 'functiongraph'
    },
    'gaussdb': {
        'service_type': 'gaussdb',
        'append_project_id': True,
        'endpoint_service_type': 'gaussdb-mysql'
    },
    'geminidb': {
        'service_type': 'geminidb',
        'append_project_id': True,
        'endpoint_service_type': 'geminidb'
    },
    'hss': {
        'service_type': 'hss',
        'append_project_id': True,
        'endpoint_service_type': 'hss'
    },
    'identity': {
        'service_type': 'identity',
        'replace_system': True
    },
    'imsv1': {
        'service_type': 'imsv1',
    },
    'imsv2': {
        'service_type': 'imsv2',
    },
    'kms': {
        'service_type': 'kms',
        'append_project_id': True,
    },
    'lts': {
        'service_type': 'lts',
        'endpoint_service_type': 'ltsv2',
        'append_project_id': True,
    },
    'mrs': {
        'service_type': 'mrs',
        'append_project_id': True,
    },
    'modelartsv1': {
        'service_type': 'modelartsv1',
        'endpoint_service_type': 'modelartsv1'
    },
    'modelartsv2': {
        'service_type': 'modelartsv2',
        'endpoint_service_type': 'modelartsv2'
    },
    'rts': {
        'service_type': 'rts',
        'endpoint_service_type': 'orchestration'
    },
    'rms': {
        'service_type': 'rms'
    },
    'nat': {
        'service_type': 'nat',
    },
    'obs': {
        'service_type': 'obs',
        'require_ak': True,
        'endpoint_service_type': 'object',
        'set_endpoint_override': True
    },
    'ocr': {
        'service_type': 'ocr',
    },
    'plas': {
        'service_type': 'plas'
    },
    'rds': {
        'service_type': 'rds',
        'endpoint_service_type': 'rdsv3',
        'append_project_id': True
    },
    'sdrs': {
        'service_type': 'sdrs',
        'append_project_id': True
    },
    'sfsturbo': {
        'service_type': 'sfsturbo',
        'endpoint_service_type': 'sfsturbo',
        # 'append_project_id': True,
    },
    'smn': {
        'service_type': 'smn',
        'append_project_id': True
    },
    'swr': {
        'service_type': 'swr',
        'endpoint_service_type': 'swrv2'
    },
    'tms': {
        'service_type': 'tms'
    },
    'vlb': {
        'service_type': 'vlb',
        'endpoint_service_type': 'elbv3',
    },
    'volume_backup': {
        'service_type': 'volume_backup',
        'append_project_id': True,
        'endpoint_service_type': 'vbs',
    },
    'vpc': {
        'service_type': 'vpc',
        'append_project_id': True,
    },
    'vpcep': {
        'service_type': 'vpcep',
        'append_project_id': True,
    },
    'vpn': {
        'service_type': 'vpn',
        'endpoint_service_type': 'vpc2.0',
        'append_project_id': True,
    },
    'waf': {
        'service_type': 'waf',
        # 'set_endpoint_override': True
    },
    'wafd': {
        'service_type': 'wafd',
        'endpoint_service_type': 'premium-waf'
    }
}

# Specifies the number of seconds a temporary AK/SK
# is to be renewed before it expires
# Usually 30 seconds should be sufficient
TMP_AKSK_RENEWAL = 30


def _get_descriptor(service_name):
    """Find ServiceDescriptor class by the service name
    and instanciate it
    """
    service = OTC_SERVICES.get(service_name, None)
    if service:
        service_type = service['service_type']

        desc_class = _find_service_description_class(service_type)
        # _logger.debug('descriptor class %s' % desc_class)
        descriptor_args = {
            'service_type': service.get('endpoint_service_type', service_type),
            'aliases': [service.get('service_type', service_type)]
        }

        if not desc_class.supported_versions:
            doc = _DOC_TEMPLATE.format(
                class_name="{service_type} Proxy".format(
                    service_type=service_type),
                **service)
        elif len(desc_class.supported_versions) == 1:
            supported_version = list(
                desc_class.supported_versions.keys())[0]
            doc = _DOC_TEMPLATE.format(
                class_name="{service_type} Proxy <{name}>".format(
                    service_type=service_type, name=supported_version),
                **service)
        else:
            class_doc_strings = "\n".join([
                ":class:`{class_name}`".format(
                    class_name=proxy_class.__name__)
                for proxy_class in desc_class.supported_versions.values()])
            doc = _PROXY_TEMPLATE.format(
                class_doc_strings=class_doc_strings, **service)
        descriptor = desc_class(**descriptor_args)
        descriptor.__doc__ = doc

        # _logger.debug('proxy is %s' % descriptor.proxy_class)

        return descriptor
    else:
        _logger.warn('unknown service %s was requested' % service_name)
        return None


def _find_service_description_class(service_type):
    package_name = 'otcextensions.sdk.{service_type}'.format(
        service_type=service_type).replace('-', '_')
    module_name = service_type.replace('-', '_') + '_service'
    class_name = ''.join(
        [part.capitalize() for part in module_name.split('_')])
    # try:
    import_name = '.'.join([package_name, module_name])
    service_description_module = importlib.import_module(import_name)
    # except ImportError as e:
    #     # ImportWarning is ignored by default. This warning is here
    #     # as an opt-in for people trying to figure out why something
    #     # didn't work.
    #     _logger.warn("Could not import {service_type} "
    #                  "service description: {e}".format(
    #                     service_type=service_type, e=str(e)),
    #                  ImportWarning)
    #     return service_description.ServiceDescription
    # There are no cases in which we should have a module but not the class
    # inside it.
    service_description_class = getattr(service_description_module, class_name)
    return service_description_class


def _get_ak_sk_autocreate(conn):
    """Autocreate/Return a temporary AK/SK"""
    config_tmp_aksk = conn.config.config.get('_tmp_aksk', {})

    # Check if a new temporary AK/SK needs to be created
    if (config_tmp_aksk.get('expires', 0) - TMP_AKSK_RENEWAL) \
            < int(time.time()):
        # Create new temporary AK/SK with minimal duration of 900s
        tmp_aksk = conn.identity.create_security_token(duration=900)
        config_tmp_aksk['expires'] = int(time.time())
        config_tmp_aksk['access_key'] = tmp_aksk.access
        config_tmp_aksk['secret_key'] = tmp_aksk.secret
        config_tmp_aksk['security_token'] = tmp_aksk.security_token
        # Save temporary AK/SK in config
        conn.config.config['_tmp_aksk'] = config_tmp_aksk

    # Retrieve temporary AK/SK from config
    ak = config_tmp_aksk['access_key']
    sk = config_tmp_aksk['secret_key']
    token = config_tmp_aksk['security_token']
    return ak, sk, token


def get_ak_sk(conn):
    """Fetch AK/SK from the cloud configuration or ENV

      This method might be called by the proxy.
    """
    config = conn.config.config

    ak = config.get('access_key', config.get('ak'))
    sk = config.get('secret_key', config.get('sk'))
    token = config.get('security_token', config.get('token'))

    if not ak:
        ak = os.getenv('OS_ACCESS_KEY', os.getenv('S3_ACCESS_KEY_ID'))
    if not sk:
        sk = os.getenv('OS_SECRET_KEY', os.getenv('S3_SECRET_ACCESS_KEY'))
    if not token:
        token = os.getenv('OS_SECURITY_TOKEN', os.getenv('S3_SECURITY_TOKEN'))

    # Autocreate/return a temporary AK/SK if "autocreate_aksk" is enabled
    if not (ak and sk) and config.get('autocreate_aksk', False):
        (ak, sk, token) = _get_ak_sk_autocreate(conn)

    if not (ak and sk):
        _logger.error('AK/SK pair is not configured in the connection, '
                      'but is requested by the service.')
        return (None, None)

    else:
        if not token:
            return ak, sk
        else:
            return ak, sk, token


def extend_instance(obj, cls):
    """Apply mixins to a class instance after creation"""
    base_cls = obj.__class__
    base_cls_name = obj.__class__.__name__
    obj.__class__ = type(base_cls_name, (base_cls, cls), {})


def patch_openstack_resources():
    openstack.proxy.Proxy._report_stats_statsd = \
        proxy.Proxy._report_stats_statsd
    openstack.proxy.Proxy._report_stats_influxdb = \
        proxy.Proxy._report_stats_influxdb
    openstack.compute.v2.server.Server._get_tag_struct = \
        server.Server._get_tag_struct
    openstack.compute.v2.server.Server._prepare_override_url = \
        server.Server._prepare_override_url
    openstack.compute.v2.server.Server.add_tag = server.Server.add_tag
    openstack.compute.v2.server.Server.remove_tag = server.Server.remove_tag
    openstack.exceptions.raise_from_response = \
        exc.raise_from_response
    openstack.network.v2.service_provider.ServiceProvider.list = \
        service_provider.ServiceProvider.list


def register_single_service(conn, service_name, project_id=None, service=None):
    """Register single service in the SDK"""
    if not project_id:
        project_id = conn._get_project_info().id
    if not service and service_name in OTC_SERVICES:
        service = OTC_SERVICES[service_name]

    if service.get('replace_system', False):
        # system_proxy = getattr(conn, service['service_type'])
        # for service_type in system_proxy.all_types:
        if service['service_type'] in conn._proxies:
            del conn._proxies[service['service_type']]
        # attr = getattr(conn, service_name)
        # delattr(conn, service['service_type'])

    sd = _get_descriptor(service_name)

    conn.add_service(sd)

    if service.get('append_project_id', False):
        # If service requires project_id, but it is not present in the
        # service catalog - set endpoint_override
        ep = conn.endpoint_for(sd.service_type)
        if ep and not ep.rstrip('/').endswith('\\%(project_id)s') \
                and not ep.rstrip('/').endswith('$(tenant_id)s') \
                and not ep.rstrip('/').endswith(project_id):
            key = '_'.join([
                sd.service_type.lower().replace('-', '_'),
                'endpoint_override'])
            if key not in conn.config.config:
                conn.config.config[key] = utils.urljoin(ep,
                                                        '%(project_id)s')

    elif service.get('set_endpoint_override', False):
        # SDK respects skip_discovery only if endpoint_override is set.
        # In case, when append_project_id is skipped for the service,
        # but the discovery on the service is not working - we might be
        # failing dramatically.
        ep = conn.endpoint_for(sd.service_type)
        conn.config.config[
            '_'.join([
                sd.service_type.lower().replace('-', '_'),
                'endpoint_override'
            ])
        ] = utils.urljoin(ep)

    # Inject get_ak_sk into the connection to give possibility
    # for some proxies to use them
    setattr(conn, 'get_ak_sk', get_ak_sk)


def load(conn, **kwargs):
    """Register supported OTC services and make them known to the OpenStackSDK

    :param conn: An established OpenStack cloud connection

    :returns: none
    """
    conn.authorize()
    project_id = conn._get_project_info().id

    for (service_name, service) in OTC_SERVICES.items():
        # _logger.debug('trying to register service %s' % service_name)
        register_single_service(conn, service_name, project_id, service)

    patch_openstack_resources()

    extend_instance(conn, _rds.RdsMixin)
    extend_instance(conn, _cce.CceMixin)
    extend_instance(conn, _dds.DdsMixin)

    return None


register_otc_extensions = load



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts data import task v2 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import cli_utils
from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {}
    hidden = ["location"]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


class ListDatasetImportTasks(command.Lister):
    _description = _("Get properties of a vm")
    columns = ("Task Id", "Created At", "Status")

    def get_parser(self, prog_name):
        parser = super(ListDatasetImportTasks, self).get_parser(prog_name)
        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            help=_("Dataset Id or Name."),
        )
        parser.add_argument(
            "--limit",
            metavar="<limit>",
            type=int,
            help=_(
                "Maximum number of records returned on each "
                "page. The default value is 10."
            ),
        )
        parser.add_argument(
            "--offset",
            metavar="<offset>",
            type=int,
            help=_("Start page of the paging list. The default value is 0."),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2

        dataset = client.find_dataset(
            parsed_args.dataset, ignore_missing=False
        )

        query = {}
        if parsed_args.limit:
            query["limit"] = parsed_args.limit
        if parsed_args.offset:
            query["offset"] = parsed_args.offset

        data = client.dataset_import_tasks(dataset, **query)

        formatters = {
            "Created At": cli_utils.UnixTimestampFormatter,
        }

        return (
            self.columns,
            (
                utils.get_dict_properties(
                    s, self.columns, formatters=formatters
                )
                for s in data
            ),
        )


class ShowDatasetImportTask(command.ShowOne):
    _description = _("Show details of a MA dataset import task")

    def get_parser(self, prog_name):
        parser = super(ShowDatasetImportTask, self).get_parser(prog_name)

        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            help=_("Enter dataset name or Id."),
        )

        parser.add_argument(
            "taskId",
            metavar="<taskId>",
            help=_("Enter task id."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2

        dataset = client.find_dataset(
            parsed_args.dataset, ignore_missing=False
        )

        obj = client.get_dataset_import_task(dataset, parsed_args.taskId)
        display_columns, columns = _get_columns(obj)
        formatters = {
            "created_at": cli_utils.UnixTimestampFormatter,
            "updated_at": cli_utils.UnixTimestampFormatter,
        }
        data = utils.get_item_properties(obj, columns, formatters=formatters)

        return (display_columns, data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import mock

from otcextensions.osclient.dcs.v1 import quota
from otcextensions.tests.unit.osclient.dcs.v1 import fakes


class TestDCSQuota(fakes.TestDCS):

    def setUp(self):
        super(TestDCSQuota, self).setUp()
        self.client = self.app.client_manager.dcs


class TestListDCSQuota(TestDCSQuota):
    quotas = fakes.FakeQuota.create_multiple(2)
    columns = ('quota', 'used', 'type', 'min', 'max', 'unit')

    data = []

    for s in quotas:
        data.append((
            s.quota,
            s.used,
            s.type,
            s.min,
            s.max,
            s.unit
        ))

    def setUp(self):
        super(TestListDCSQuota, self).setUp()
        self.cmd = quota.ListDCSQuota(self.app, None)
        self.client.quotas = mock.Mock()

    def test_list_quota(self):
        arglist = [
        ]

        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.quotas.side_effect = [
            self.quotas
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.quotas.assert_called_once_with()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource


class JobEngine(resource.Resource):
    base_path = "/job/ai-engines"

    resources_key = "engines"

    _query_mapping = resource.QueryParameters(
        "job_type",
    )

    # capabilities
    allow_list = True

    # Properties
    #: ID of the engine selected for a training job.
    engine_id = resource.Body("engine_id", type=int)
    #: Name of the engine selected for a training job.
    engine_name = resource.Body("engine_name")
    #: Engine type of a training job1: TensorFlow5: Spark_MLlib6: Scikit
    #:  Learn9: XGBoost-Sklearn10: PyTorch17: MindSpore-GPU.
    engine_type = resource.Body("engine_type", type=int)
    #: Version of the engine selected for a training job.
    engine_version = resource.Body("engine_version")
    #: Whether the request is successful.
    is_success = resource.Body("is_success", type=bool)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import uuid
import random

from openstackclient.tests.functional import base


class DnsTests(base.TestCase):
    """Functional tests for DNS. """

    NAME = uuid.uuid4().hex
    OTHER_NAME = uuid.uuid4().hex

    def setUp(self):
        super(DnsTests, self).setUp()
        self.ZONE_NAME = 'example-{0}.org.'.format(random.randint(1, 10000))

    def test_zone_list(self):
        json_output = json.loads(self.openstack(
            'dns zone create {name} -f json'.format(name=self.ZONE_NAME)
        ))
        self.zone_id = json_output["id"]
        self.assertOutput(self.ZONE_NAME, json_output['name'])
        self.addCleanup(self.openstack, 'dns zone delete {name}'.format(
            name=self.ZONE_NAME))

        json_output = json.loads(self.openstack(
            'dns zone list -f json '
        ))
        self.assertIn(
            self.ZONE_NAME,
            [zone['name'] for zone in json_output]
        )

        json_output = json.loads(self.openstack(
            'dns zone show {name} -f json'.format(name=self.ZONE_NAME)
        ))
        self.assertEqual(
            self.zone_id,
            json_output['id']
        )

        json_output = json.loads(self.openstack(
            'dns zone set --description test {name} -f json'.format(
                name=self.ZONE_NAME)
        ))

        self.assertEqual(
            'test',
            json_output['description']
        )

        rs_name = '{a}.{name}'.format(
            a=random.randint(1, 10000),
            name=self.ZONE_NAME
        )
        json_output = json.loads(self.openstack(
            'dns recordset create --name {rs_name} '
            '--type A --record 127.0.0.1 {zone_name} -f json'.format(
                zone_name=self.ZONE_NAME,
                rs_name=rs_name)
        ))

        self.assertOutput(rs_name, json_output['name'])
        self.assertIn('127.0.0.1', json_output['records'])

        rsid = json_output['id']

        self.addCleanup(self.openstack,
                        'dns recordset delete {z} {r}'.format(
                            r=rsid,
                            z=self.ZONE_NAME))

        json_output = json.loads(self.openstack(
            'dns recordset show {z} {r} -f json'.format(
                r=rsid,
                z=self.ZONE_NAME
            )
        ))

        self.assertEqual(rs_name, json_output['name'])

        json_output = json.loads(self.openstack(
            'dns recordset list {zone} -f json'.format(
                zone=self.ZONE_NAME)
        ))

        self.assertIn(
            rs_name,
            [rec['name'] for rec in json_output]
        )

        json_output = json.loads(self.openstack(
            'dns recordset set --record 192.168.0.1 {z} {r} -f json'.format(
                z=self.ZONE_NAME,
                r=rsid)
        ))

        self.assertIn('192.168.0.1', json_output['records'])

        net_id = self.openstack(
            'network list --name admin_external_net -f value -c ID')

        json_output = json.loads(self.openstack(
            'floating ip create {net} -f json'.format(
                net=net_id)
        ))

        fip_id = json_output['id']

        self.addCleanup(self.openstack,
                        'floating ip delete {fip}'.format(
                            fip=fip_id))

        ptr_fip = 'eu-de:' + fip_id
        json_output = json.loads(self.openstack(
            'dns ptr record set {id} {z} -f json'.format(
                id=ptr_fip,
                z=rs_name)))

        self.assertEqual(rs_name, json_output['ptrdname'])

        self.addCleanup(self.openstack,
                        'dns ptr record unset {id}'.format(
                            id=ptr_fip))

        json_output = json.loads(self.openstack(
            'dns ptr record show {ptr} -f json'.format(
                ptr=ptr_fip)))

        self.assertEqual(rs_name, json_output['ptrdname'])

        json_output = json.loads(self.openstack(
            'dns ptr record list -f json'))

        self.assertIn(
            rs_name,
            [rec['ptrdname'] for rec in json_output]
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from otcextensions.sdk.ctsv3.v3 import _proxy
from otcextensions.sdk.ctsv3.v3 import key_event as _key_event
from otcextensions.sdk.ctsv3.v3 import trace as _trace
from otcextensions.sdk.ctsv3.v3 import tracker as _tracker
from otcextensions.sdk.ctsv3.v3 import quota as _quota

from openstack.tests.unit import test_proxy_base


class TestCTSv3Proxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestCTSv3Proxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestKeyEvent(TestCTSv3Proxy):
    def test_key_event_create(self):
        self.verify_create(
            self.proxy.create_key_event,
            _key_event.KeyEvent
        )

    def test_key_event_update(self):
        self.verify_update(
            self.proxy.update_key_event,
            _key_event.KeyEvent,
            method_args=[],
            expected_args=['']
        )

    def test_key_event_list(self):
        self.verify_list(
            self.proxy.key_events,
            _key_event.KeyEvent,
            method_kwargs={'notification_type': 'type'},
            expected_kwargs={},
        )


class TestTraces(TestCTSv3Proxy):
    def test_trace_list(self):
        self.verify_list(
            self.proxy.traces,
            _trace.Trace
        )


class TestTracker(TestCTSv3Proxy):
    def test_tracker_list(self):
        self.verify_list(
            self.proxy.trackers,
            _tracker.Tracker
        )

    def test_tracker_create(self):
        self.verify_create(
            self.proxy.create_tracker,
            _tracker.Tracker
        )

    def test_tracker_update(self):
        self._verify(
            "openstack.proxy.Proxy._update",
            self.proxy.update_tracker,
            method_args=[],
            method_kwargs={"x": 1, "y": 2, "z": 3},
            expected_args=[_tracker.Tracker],
            expected_kwargs={"x": 1, "y": 2, "z": 3},
        )


class TestQuota(TestCTSv3Proxy):
    def test_quota_list(self):
        self.verify_list(
            self.proxy.quotas,
            _quota.Quota
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from keystoneauth1 import adapter
import mock
from openstack.tests.unit import base

from otcextensions.sdk.cbr.v3 import backup as _backup


EXAMPLE = {
    'provider_id': uuid.uuid4(),
    'checkpoint_id': uuid.uuid4(),
    'updated_at': '2020-02-21T07:07:25.113761',
    'vault_id': uuid.uuid4(),
    'id': uuid.uuid4(),
    'resource_az': 'eu-de-01',
    'image_type': 'backup',
    'resource_id': uuid.uuid4(),
    'resource_size': 40,
    'children': [],
    'extend_info': {
        'auto_trigger': True,
        'supported_restore_mode': 'backup',
        'contain_system_disk': True,
        'support_lld': True,
        'architecture': 'x86_64',
        'system_disk': False,
        'app_consistency': {
            'error_code': '0',
            'error_status': '0',
            'error_message': '',
            'app_consistency': '0'
        }
    },
    'project_id': uuid.uuid4(),
    'status': 'available',
    'resource_name': 'test001-02',
    'description': '',
    'expired_at': '2020-05-21T07:00:54.060493',
    'replication_records': [],
    'name': 'autobk_b629',
    'created_at': '2020-02-21T07:00:54.065135',
    'resource_type': 'OS::Nova::Server'
}


class TestBackup(base.TestCase):

    def setUp(self):
        super(TestBackup, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.delete = mock.Mock()
        self.sess.put = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)
        self.sot = _backup.Backup()
        self.sot_expected = _backup.Backup(**EXAMPLE)

    def test_basic(self):
        sot = _backup.Backup()
        self.assertEqual('backup', sot.resource_key)
        self.assertEqual('backups', sot.resources_key)
        self.assertEqual('/backups',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertDictEqual({
            'checkpoint_id': 'checkpoint_id',
            'dec': 'dec',
            'end_time': 'end_time',
            'image_type': 'image_type',
            'limit': 'limit',
            'marker': 'marker',
            'member_status': 'member_status',
            'name': 'name',
            'offset': 'offset',
            'own_type': 'own_type',
            'parent_id': 'parent_id',
            'resource_az': 'resource_az',
            'resource_id': 'resource_id',
            'resource_name': 'resource_name',
            'resource_type': 'resource_type',
            'sort': 'sort',
            'start_time': 'start_time',
            'status': 'status',
            'used_percent': 'used_percent',
            'vault_id': 'vault_id'
        }, sot._query_mapping._mapping)

    def test_make_it(self):
        test_backup = _backup.Backup(**EXAMPLE)
        self.assertEqual(
            EXAMPLE['provider_id'],
            test_backup.provider_id)
        self.assertEqual(
            EXAMPLE['checkpoint_id'],
            test_backup.checkpoint_id)
        self.assertEqual(
            EXAMPLE['updated_at'],
            test_backup.updated_at)
        self.assertEqual(
            EXAMPLE['vault_id'],
            test_backup.vault_id)
        self.assertEqual(
            EXAMPLE['id'],
            test_backup.id)
        self.assertEqual(
            EXAMPLE['resource_az'],
            test_backup.resource_az)
        self.assertEqual(
            EXAMPLE['image_type'],
            test_backup.image_type)
        self.assertEqual(
            EXAMPLE['resource_id'],
            test_backup.resource_id)
        self.assertEqual(
            EXAMPLE['resource_size'],
            test_backup.resource_size)
        self.assertEqual(
            EXAMPLE['children'],
            test_backup.children)
        self.assertEqual(
            EXAMPLE['project_id'],
            test_backup.project_id)
        self.assertEqual(
            EXAMPLE['status'],
            test_backup.status)
        self.assertEqual(
            EXAMPLE['resource_name'],
            test_backup.resource_name)
        self.assertEqual(
            EXAMPLE['description'],
            test_backup.description)
        self.assertEqual(
            EXAMPLE['expired_at'],
            test_backup.expired_at)
        self.assertEqual(
            EXAMPLE['replication_records'],
            test_backup.replication_records)
        self.assertEqual(
            EXAMPLE['name'],
            test_backup.name)
        self.assertEqual(
            EXAMPLE['created_at'],
            test_backup.created_at)
        self.assertEqual(
            EXAMPLE['resource_type'],
            test_backup.resource_type)
        self.assertTrue(
            test_backup.extend_info.auto_trigger)
        self.assertEqual(
            EXAMPLE['extend_info']['supported_restore_mode'],
            test_backup.extend_info.supported_restore_mode)
        self.assertTrue(
            test_backup.extend_info.contain_system_disk)
        self.assertTrue(
            test_backup.extend_info.support_lld)
        self.assertEqual(
            EXAMPLE['extend_info']['architecture'],
            test_backup.extend_info.architecture)
        self.assertFalse(
            test_backup.extend_info.system_disk)
        self.assertEqual(
            EXAMPLE['extend_info']['app_consistency']['app_consistency'],
            test_backup.extend_info.app_consistency.app_consistency)
        self.assertEqual(
            EXAMPLE['extend_info']['app_consistency']['error_code'],
            test_backup.extend_info.app_consistency.error_code)
        self.assertEqual(
            EXAMPLE['extend_info']['app_consistency']['error_message'],
            test_backup.extend_info.app_consistency.error_message)
        self.assertEqual(
            EXAMPLE['extend_info']['app_consistency']['error_status'],
            test_backup.extend_info.app_consistency.error_status)

    def test_add_members(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        sot = _backup.Backup.existing(id=EXAMPLE['id'])

        members = ['member1', 'member2']

        sot.add_members(self.sess, members)

        self.sess.post.assert_called_once_with(
            'backups/%s/members' % EXAMPLE['id'],
            json={'members': members}
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from keystoneauth1 import adapter
import mock
from openstack.tests.unit import base

from otcextensions.sdk.sdrs.v1 import protected_instance as _protected_instance


EXAMPLE = {
    'id': uuid.uuid4(),
    'name': uuid.uuid4(),
    'description': uuid.uuid4(),
    'server_group_id': uuid.uuid4(),
    'status': 'protected',
    'progress': 0,
    'source_server': uuid.uuid4(),
    'target_server': uuid.uuid4(),
    'created_at': '2022-02-26 20:39:30.718',
    'updated_at': '2022-02-26 21:02:45.507',
    'priority_station': 'source',
    'attachment': [
        {
            'replication': uuid.uuid4(),
            'device': uuid.uuid4()
        }
    ],
    'tags': [
        {
            'key': uuid.uuid4(),
            'value': uuid.uuid4()
        }
    ],
    'metadata': {
        '__system__frozen': False
    }
}


class TestProtectedInstance(base.TestCase):

    def setUp(self):
        super(TestProtectedInstance, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)
        self.sot = _protected_instance.ProtectedInstance()

    def test_basic(self):
        sot = _protected_instance.ProtectedInstance()
        self.assertEqual('protected_instance', sot.resource_key)
        self.assertEqual('protected_instances', sot.resources_key)
        self.assertEqual('/protected-instances',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertDictEqual({
            'availability_zone': 'availability_zone',
            'limit': 'limit',
            'marker': 'marker',
            'name': 'name',
            'offset': 'offset',
            'protected_instance_ids': 'protected_instance_ids',
            'query_type': 'query_type',
            'server_group_id': 'server_group_id',
            'server_group_ids': 'server_group_ids',
            'status': 'status'
        }, sot._query_mapping._mapping)

    def test_make_it(self):
        test_protected_instance = \
            _protected_instance.ProtectedInstance(**EXAMPLE)
        self.assertEqual(
            EXAMPLE['id'],
            test_protected_instance.id)
        self.assertEqual(
            EXAMPLE['name'],
            test_protected_instance.name)
        self.assertEqual(
            EXAMPLE['description'],
            test_protected_instance.description)
        self.assertEqual(
            EXAMPLE['server_group_id'],
            test_protected_instance.server_group_id)
        self.assertEqual(
            EXAMPLE['status'],
            test_protected_instance.status)
        self.assertEqual(
            EXAMPLE['progress'],
            test_protected_instance.progress)
        self.assertEqual(
            EXAMPLE['source_server'],
            test_protected_instance.source_server)
        self.assertEqual(
            EXAMPLE['target_server'],
            test_protected_instance.target_server)
        self.assertEqual(
            EXAMPLE['created_at'],
            test_protected_instance.created_at)
        self.assertEqual(
            EXAMPLE['updated_at'],
            test_protected_instance.updated_at)
        self.assertEqual(
            EXAMPLE['priority_station'],
            test_protected_instance.priority_station)
        self.assertEqual(
            EXAMPLE['attachment'][0]['replication'],
            test_protected_instance.attachment[0].replication)
        self.assertEqual(
            EXAMPLE['attachment'][0]['device'],
            test_protected_instance.attachment[0].device)
        self.assertEqual(
            EXAMPLE['tags'][0]['key'],
            test_protected_instance.tags[0].key)
        self.assertEqual(
            EXAMPLE['tags'][0]['value'],
            test_protected_instance.tags[0].value)
        self.assertEqual(
            EXAMPLE['metadata']['__system__frozen'],
            test_protected_instance.metadata.system_frozen)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''AS Activity Log v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


class ListAutoScalingActivityLogs(command.Lister):
    _description = _('List AutoScaling Activity Logs')
    columns = (
        'ID', 'status', 'description',
        'instance_value', 'desire_value',
        'start_time', 'end_time')

    def get_parser(self, prog_name):
        parser = super(ListAutoScalingActivityLogs, self).get_parser(prog_name)
        parser.add_argument(
            '--group',
            metavar='<group>',
            required=True,
            help=_('AS Group ID or name')
        )
        parser.add_argument(
            '--start-time',
            metavar='<start_time>',
            help=_('Specifies the start time for querying scaling '
                   'action logs. Format: `YYYY-MM-DDThh:mm:ssZ`')
        )
        parser.add_argument(
            '--end-time',
            metavar='<end_time>',
            help=_('Specifies the end time for querying scaling '
                   'action logs. Format: `YYYY-MM-DDThh:mm:ssZ`')
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Specifies the batch size of the records for pagination')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.auto_scaling

        args = {}
        if parsed_args.limit:
            args['limit'] = parsed_args.limit
        if parsed_args.start_time:
            args['start_time'] = parsed_args.start_time
        if parsed_args.end_time:
            args['end_time'] = parsed_args.end_time

        group = client.find_group(parsed_args.group, ignore_missing=False)

        data = client.activities(group=group.id, **args)

        return (
            self.columns,
            (utils.get_item_properties(
                s,
                self.columns,
            ) for s in data)
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CBR Policy CLI implementation'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _flatten_backup(obj):
    """Flatten the structure of the backup into a single dict
    """
    data = {
        'id': obj.id,
        'name': obj.name,
        'checkpoint_id': obj.checkpoint_id,
        'created_at': obj.created_at,
        'description': obj.description,
        'expired_at': obj.expired_at,
        'image_type': obj.image_type,
        'parent_id': obj.parent_id,
        'project_id': obj.project_id,
        'protected_at': obj.protected_at,
        'resource_az': obj.resource_az,
        'resource_id': obj.resource_id,
        'resource_name': obj.resource_name,
        'resource_size': obj.resource_size,
        'resource_type': obj.resource_type,
        'status': obj.status,
        'updated_at': obj.updated_at,
        'vault_id': obj.vault_id,
        'provider_id': obj.provider_id
    }

    return data


def _add_children_to_backup_obj(obj, data, columns):
    """Add children to column and data tuples
    """
    i = 0
    for s in obj.children:
        if obj.children[i]['id']:
            name = 'child_backup_' + str(i + 1)
            data += (obj.children[i]['id'],)
            columns = columns + (name,)
            i += 1
    return data, columns


def _normalize_mappings(mappings):
    result = []
    for m in mappings:
        res = dict(map(lambda s: s.split('='), m.split(' ')))
        result.append(res)
    return result


class ListBackups(command.Lister):
    _description = _('List CBR Backups')
    columns = (
        'id',
        'name',
        'checkpoint_id',
        'created_at',
        'description',
        'expired_at',
        'image_type',
        'parent_id',
        'project_id',
        'protected_at',
        'resource_az',
        'resource_id',
        'resource_name',
        'resource_size',
        'resource_type',
        'status',
        'updated_at',
        'vault_id',
        'provider_id'
    )

    def get_parser(self, prog_name):
        parser = super(ListBackups, self).get_parser(prog_name)
        parser.add_argument(
            '--checkpoint-id',
            metavar='<checkpoint_id>',
            help=_('The ID of the restore point.')
        )
        parser.add_argument(
            '--dec',
            metavar='<dec>',
            type=bool,
            help=_('Dedicated cloud')
        )
        parser.add_argument(
            '--end-time',
            metavar='<end_time>',
            help=_('Time when the backup ends.')
        )
        parser.add_argument(
            '--image-type',
            metavar='<image_type>',
            choices=['backup', 'replication'],
            help=_('Backup type')
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Number of records displayed per page.')
        )
        parser.add_argument(
            '--marker',
            metavar='<marker>',
            help=_('ID of the last record displayed on the previous page.')
        )
        parser.add_argument(
            '--member-status',
            metavar='<member_status>',
            choices=['pending', 'accept', 'reject'],
            help=_('The ID of the restore point.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Backup name.')
        )
        parser.add_argument(
            '--offset',
            type=int,
            metavar='<offset>',
            help=_('Offset value. The value must be a positive integer.')
        )
        parser.add_argument(
            '--own-type',
            metavar='<own_type>',
            choices=['all_granted', 'private', 'shared'],
            help=_('Owning type of a backup. private backups are queried'
                   ' by default.')
        )
        parser.add_argument(
            '--parent-id',
            metavar='<parent_id>',
            help=_('Parent backup ID.')
        )
        parser.add_argument(
            '--resource-az',
            metavar='<resource_az>',
            help=_('AZ-based filtering is supported.')
        )
        parser.add_argument(
            '--resource-id',
            metavar='<resource_id>',
            help=_('Resource ID.')
        )
        parser.add_argument(
            '--resource-name',
            metavar='<resource_name>',
            help=_('Resource name.')
        )
        parser.add_argument(
            '--resource-type',
            metavar='<resource_type>',
            choices=['OS::Cinder::Volume', 'OS::Nova::Server'],
            help=_('Resource type.')
        )
        parser.add_argument(
            '--sort',
            metavar='sort',
            help=_('A group of properties separated by commas (,)'
                   ' and sorting directions.')
        )
        parser.add_argument(
            '--start-time',
            metavar='<start_time>',
            help=_('Time when the backup starts.')
        )
        parser.add_argument(
            '--status',
            metavar='status',
            choices=['available', 'protecting', 'deleting', 'restoring',
                     'error', 'waiting_protect', 'waiting_delete',
                     'waiting_restore'],
            help=_('Status')
        )
        parser.add_argument(
            '--used-percent',
            metavar='<used_percent>',
            help=_('Backups are filtered based on the '
                   'occupied vault capacity.')
        )
        parser.add_argument(
            '--vault-id',
            metavar='<vault_id>',
            help=_('Vault ID.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        query = {}

        args_list = ['checkpoint_id', 'dec', 'end_time', 'image_type', 'limit',
                     'marker', 'member_status', 'name', 'offset', 'own_type',
                     'parent_id', 'resource_az', 'resource_id',
                     'resource_name', 'resource_type', 'sort', 'start_time',
                     'status', 'used_percent', 'vault_id']

        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val is not None:
                query[arg] = val

        data = client.backups(**query)

        table = (self.columns,
                 (utils.get_dict_properties(
                     _flatten_backup(s), self.columns,
                 ) for s in data))
        return table


class ShowBackup(command.ShowOne):
    _description = _('Show single Backup details')
    columns = (
        'id',
        'name',
        'checkpoint_id',
        'created_at',
        'description',
        'expired_at',
        'image_type',
        'parent_id',
        'project_id',
        'protected_at',
        'resource_az',
        'resource_id',
        'resource_name',
        'resource_size',
        'resource_type',
        'status',
        'updated_at',
        'vault_id',
        'provider_id'
    )

    def get_parser(self, prog_name):
        parser = super(ShowBackup, self).get_parser(prog_name)
        parser.add_argument(
            'backup',
            metavar='<backup>',
            help=_('ID or name of the CBR backup.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        obj = client.find_backup(
            name_or_id=parsed_args.backup,
            ignore_missing=False
        )

        data = utils.get_dict_properties(
            _flatten_backup(obj), self.columns)

        if obj.children:
            data, self.columns = _add_children_to_backup_obj(
                obj, data, self.columns)

        return (self.columns, data)


class DeleteBackup(command.Command):
    _description = _('Delete CBR Backup')

    def get_parser(self, prog_name):
        parser = super(DeleteBackup, self).get_parser(prog_name)
        parser.add_argument(
            'backup',
            metavar='<backup>',
            help=_('ID or name of the CBR Backup.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        backup = client.find_backup(
            name_or_id=parsed_args.backup,
            ignore_missing=False
        )

        self.app.client_manager.cbr.delete_backup(
            backup=backup.id,
            ignore_missing=False)


class RestoreBackup(command.Command):
    _description = _('Restore CBR Backup data')

    columns = (
        'id',
        'name',
        'checkpoint_id',
        'created_at',
        'description',
        'expired_at',
        'image_type',
        'parent_id',
        'project_id',
        'protected_at',
        'resource_az',
        'resource_id',
        'resource_name',
        'resource_size',
        'resource_type',
        'status',
        'updated_at',
        'vault_id',
        'provider_id',
    )

    def get_parser(self, prog_name):
        parser = super(RestoreBackup, self).get_parser(prog_name)
        parser.add_argument(
            'backup',
            metavar='backup',
            help=_('Backup id.')
        )
        parser.add_argument(
            '--mappings',
            metavar='<mappings>',
            action='append',
            help=_('Restored mapping relationship  in "volume_id=volume_uuid'
                   ' backup_id=backup_uuid", where volume_id - ID of the disk'
                   ' to which data is restored, backup_id - Disk backup ID.')
        )
        parser.add_argument(
            '--power-on',
            action='store_true',
            help=_('Whether the server is powered on after restoration.')
        )
        parser.add_argument(
            '--server-id',
            metavar='<server_id>',
            help=_('ID of the target VM to be restored. This parameter is'
                   ' mandatory for VM restoration.')
        )
        parser.add_argument(
            '--volume-id',
            metavar='<volume_id>',
            help=_('ID of the target disk to be restored. This parameter is'
                   ' mandatory for disk restoration.')
        )

        return parser

    def take_action(self, parsed_args):

        query = {
            'mappings': []
        }

        backup = self.app.client_manager.cbr.find_backup(
            name_or_id=parsed_args.backup,
            ignore_missing=False
        )

        args_list = ['power_on', 'server_id', 'volume_id']

        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val is not None:
                query[arg] = val

        if parsed_args.mappings:
            query['mappings'] = _normalize_mappings(parsed_args.mappings)

        self.app.client_manager.cbr.restore_data(backup=backup.id, **query)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter

from openstack.tests.unit import base

from otcextensions.sdk.rds.v3 import backup

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'id': IDENTIFIER,
    'name': '50deafb3e45d451a9406ca146b71fe9a_rds',
    'description': '',
    'begin_time': '2016-08-23T04:01:40',
    'end_time': '2016-08-23T04:04:40',
    'status': 'COMPLETED',
    'instance_id': '4f87d3c4-9e33-482f-b962-e23b30d1a18c',
    'type': 'manual',
    'datastore': {}
}

EXAMPLE_FILE = {
    'name': '43e4feaab48f11e89039fa163ebaa7e4br01.xxx',
    'size': 2803,
    'download_link': 'fake_url',
    'link_expired_time': '2018-08-016T10:15:14+0800'
}


class TestBackup(base.TestCase):

    def setUp(self):
        super(TestBackup, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.default_microversion = None
        self.sess._get_connection = mock.Mock(return_value=self.cloud)

    def test_basic(self):
        sot = backup.Backup()
        self.assertEqual('backup', sot.resource_key)
        self.assertEqual('backups', sot.resources_key)
        self.assertEqual('/backups', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = backup.Backup(**EXAMPLE)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['instance_id'], sot.instance_id)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['begin_time'], sot.begin_time)
        self.assertEqual(EXAMPLE['end_time'], sot.end_time)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['datastore'], sot.datastore)

    def test_fetch(self):
        sot = backup.Backup(**EXAMPLE)

        response = mock.Mock()
        response.json.return_value = {
            'backups': [EXAMPLE.copy()]
        }
        response.headers = {}
        response.status_code = 200

        self.sess.get.return_value = response

        rt = sot.fetch(self.sess)

        self.sess.get.assert_called_with(
            'backups',
            headers={'Accept': 'application/json'},
            microversion=None,
            params={'instance_id': sot.instance_id, 'backup_id': sot.id})

        self.assertDictEqual(sot.to_dict(), rt.to_dict())


class TestBackupFile(base.TestCase):

    def test_basic(self):
        sot = backup.BackupFile()
        self.assertEqual('files', sot.resources_key)
        self.assertEqual('/backup-files', sot.base_path)
        self.assertTrue(sot.allow_list)

        self.assertDictEqual({
            'limit': 'limit',
            'marker': 'marker',
            'backup_id': 'backup_id'},
            sot._query_mapping._mapping)

    def test_make_it(self):
        sot = backup.BackupFile(**EXAMPLE_FILE)
        self.assertEqual(EXAMPLE_FILE['name'], sot.name)
        self.assertEqual(EXAMPLE_FILE['size'], sot.size)
        self.assertEqual(EXAMPLE_FILE['download_link'], sot.download_link)
        self.assertEqual(EXAMPLE_FILE['link_expired_time'], sot.expires_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Log(resource.Resource):
    base_path = '/fgs/functions/%(function_urn)s/lts-log-detail'
    # Capabilities
    allow_create = True
    allow_fetch = True

    # Properties
    function_urn = resource.URI('function_urn', type=str)

    # Attributes
    #: Log group name.
    group_name = resource.Body('group_name', type=str)
    #: Log group ID.
    group_id = resource.Body('group_id', type=str)
    #: Log stream ID.
    stream_id = resource.Body('stream_id', type=str)
    #: Log stream name.
    stream_name = resource.Body('stream_name', type=str)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock

from otcextensions.osclient.cce.v2 import cluster_certificate
from otcextensions.sdk.cce.v3 import cluster
from otcextensions.tests.unit.osclient.cce.v2 import fakes


class TestClusterCertificate(fakes.TestCCE):

    def setUp(self):
        super(TestClusterCertificate, self).setUp()

    def test_flatten(self):
        _obj = fakes.FakeClusterCertificate.create_one()

        flat_data = cluster_certificate._flatten_cluster_certificate(_obj)

        data = (
            flat_data['name'],
            flat_data['cluster'],
            flat_data['user'],
            flat_data['ca'],
            flat_data['client_certificate'],
            flat_data['client_key'],
        )

        cmp_data = (
            _obj.context['name'],
            _obj.context['cluster'],
            _obj.context['user'],
            _obj.ca,
            _obj.client_certificate,
            _obj.client_key,
        )

        self.assertEqual(data, cmp_data)


class TestClusterCertificateShow(fakes.TestCCE):
    _obj = fakes.FakeClusterCertificate.create_one()

    columns = ('name', 'cluster', 'user', 'ca',
               'client_certificate', 'client_key')
    flat_data = cluster_certificate._flatten_cluster_certificate(_obj)
    data = (
        flat_data['name'],
        flat_data['cluster'],
        flat_data['user'],
        flat_data['ca'],
        flat_data['client_certificate'],
        flat_data['client_key'],
    )

    def setUp(self):
        super(TestClusterCertificateShow, self).setUp()

        self.cmd = cluster_certificate.ShowCCEClusterCertificates(
            self.app, None)

        self.client.find_cluster = mock.Mock(
            return_value=cluster.Cluster(id='cluster_uuid'))
        self.client.get_cluster_certificates = mock.Mock()

    def test_get(self):
        arglist = [
            'cluster_uuid'
        ]

        verifylist = [
            ('cluster', 'cluster_uuid')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.get_cluster_certificates.side_effect = [
            self._obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.get_cluster_certificates.assert_called_once_with(
            cluster='cluster_uuid',
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy
from openstack import resource

from otcextensions.sdk.dds.v3 import datastore as _datastore
from otcextensions.sdk.dds.v3 import flavor as _flavor
from otcextensions.sdk.dds.v3 import instance as _instance
from otcextensions.sdk.dds.v3 import job as _job
from otcextensions.sdk.dds.v3 import eip as _eip
from otcextensions.sdk.dds.v3 import recycle_policy as _recycle_policy
from otcextensions.sdk.dds.v3 import recycle_instance as _recycle_instance


class Proxy(proxy.Proxy):
    skip_discovery = True

    # ======= Datastores =======
    def datastore_types(self):
        """List supported datastore types

        :returns: A generator of supported datastore types
        """
        for ds in ['DDS-Community']:
            obj = type('obj', (object,), {'name': ds})
            yield obj

    def datastores(self, datastore_name):
        """List datastores

        :param datastore_name: database store name \
        (currently only DDS-Community and is case-sensitive.)

        :returns: A generator of supported datastore versions.

        :rtype: :class:`~otcextensions.sdk.dds.v3.datastore.Datastore`

        """
        return self._list(
            _datastore.Datastore,
            datastore_name=datastore_name,
        )

    # ======= Flavors =======
    def flavors(self, region, engine_name):
        """List flavors of all DB instances specifications in specified region

        :param engine_name: database engine name
        :param region: region

        :returns: A generator of flavor
        :rtype: :class:`~otcextensions.sdk.dds.v3.flavor.Flavor`
        """

        return self._list(
            _flavor.Flavor,
            region=region,
            engine_name=engine_name
        )

    # ======= Instance =======
    def create_instance(self, **attrs):
        """Create a new instance from attributes

        :param dict attrs: Keyword arguments which will be used to create \
        a :class:`~otcextensions.sdk.dds.v3.instance.Instance`, \
        comprised of the properties on the Instance class.

        :returns: The result of an instance creation.

        :rtype: :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        return self._create(_instance.Instance, **attrs)

    def restart_instance(self, instance):
        """Restart an existing instance

        :param instance: The value can be either the ID of an instance or a \
        :class:`~otcextensions.sdk.dds.v3.instance.Instance` instance.

        :returns: job ID.

        :rtype: :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.restart(self)

    def enlarge_instance(self, instance, size, group_id=None):
        """Enlarge storage space of a DB instance

        :param instance: The value can be either the ID of an instance or a \
        :class:`~otcextensions.sdk.dds.v3.instance.Instance` instance.
        :param size: New instance size.
        :param group_id: ID of the group to enlarge storage space.

        :returns: job ID.

        :rtype: :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        instance = self.get_instance(instance)
        return instance.enlarge(self, size, group_id)

    def add_instance_nodes(self, instance, **attrs):
        """Add nodes for a specific instance

        :param instance: The value can be either the ID of an instance or a \
        :class:`~otcextensions.sdk.dds.v3.instance.Instance` instance.
        :param dict attrs: Keyword arguments which will be used to add node \
         to a :class:`~otcextensions.sdk.dds.v3.instance.Instance`.

        :returns: job ID.

        :rtype: :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.add_nodes(self, **attrs)

    def resize_instance(self, instance, **attrs):
        """Change specifications of a DB instance

        :param instance: The value can be either the ID of an instance or a \
        :class:`~otcextensions.sdk.dds.v3.instance.Instance` instance.
        :param dict attrs: Keyword arguments which will be used to add node \
         to a :class:`~otcextensions.sdk.dds.v3.instance.Instance`.

        :returns: job ID.

        :rtype: :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.resize(self, **attrs)

    def switchover_instance(self, instance):
        """Perform a primary/secondary switchover in a replica set instance.

        :param instance: The value can be either the ID of an instance or a \
        :class:`~otcextensions.sdk.dds.v3.instance.Instance` instance.

        :returns: job ID.

        :rtype: :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.switchover(self)

    def enable_instance_ssl(self, instance, enable=True):
        """Enable SSL in a replica set instance.

        :param instance: The value can be either the ID of an instance or a \
        :class:`~otcextensions.sdk.dds.v3.instance.Instance` instance.
        :param bool enable: enable or disable

        :returns: job ID.

        :rtype: :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.switch_ssl(self, enable)

    def change_instance_name(self, instance, name):
        """Change name of a DB instance

        :param instance: The value can be either the ID of an instance or a \
        :class:`~otcextensions.sdk.dds.v3.instance.Instance` instance.
        :param name: New name of an instance

        :rtype: :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        instance = self._get_resource(_instance.Instance, instance)
        instance.modify_name(self, name)

    def change_instance_port(self, instance, port):
        """Change port of a DB instance

        :param instance: The value can be either the ID of an instance or a \
        :class:`~otcextensions.sdk.dds.v3.instance.Instance` instance.
        :param port: New port of an instance

        :returns: DB instance object.

        :rtype: :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.change_port(self, port)

    def change_instance_security_group(self, instance, security_group_id):
        """Change security group of a DB instance

        :param instance: The value can be either the ID of an instance or a \
        :class:`~otcextensions.sdk.dds.v3.instance.Instance` instance.
        :param security_group_id: New security group ID

        :returns: job ID.

        :rtype: :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.change_security_group(self, security_group_id)

    def change_instance_private_ip(self, instance, **attrs):
        """Change private IP of a DB instance

        :param instance: The value can be either the ID of an instance or a \
        :class:`~otcextensions.sdk.dds.v3.instance.Instance` instance.
        :param dict attrs: Keyword arguments which will be used to change ip.

        :returns: job ID.

        :rtype: :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.change_private_ip(self, **attrs)

    def create_instance_ip(self, instance, **attrs):
        """Add IP for a DB instance

        :param instance: The value can be either the ID of an instance or a \
        :class:`~otcextensions.sdk.dds.v3.instance.Instance` instance.
        :param dict attrs: Keyword arguments which will be used to add ip

        :returns: workflow ID.

        :rtype: :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.create_ip(self, **attrs)

    def configure_client_network(self, instance, network_ranges):
        """Configure client network of a DB instance
        :param instance: The value can be either the ID of an instance or a \
        :class:`~otcextensions.sdk.dds.v3.instance.Instance` instance.
        :param list network_ranges: List of network ranges

        :returns: workflow ID.

        :rtype: :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.configure_client_network(self, network_ranges)

    def delete_instance(self, instance, ignore_missing=True):
        """Delete an instance

        :param instance: The value can be either the ID of an instance or a \
        :class:`~otcextensions.sdk.dds.v3.instance.Instance` instance.
        :param bool ignore_missing: When set to ``False`` \
        :class:`~openstack.exceptions.ResourceNotFound` will be \
        raised when the instance does not exist. \
        When set to ``True``, no exception will be set when \
        attempting to delete a nonexistent instance.

        :returns: ``None``

        """
        return self._delete(
            _instance.Instance,
            instance,
            ignore_missing=ignore_missing,
        )

    def get_instance(self, instance):
        """Get a single instance

        :param instance: The value can be either the ID of an instance or a \
        :class:`~otcextensions.sdk.dds.v3.instance.Instance` instance.

        :returns: One :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        return self._get(_instance.Instance, instance)

    def find_instance(self, name_or_id, ignore_missing=True):
        """Find a single instance

        :param name_or_id: The name or ID of a instance.
        :param bool ignore_missing: When set to ``False`` \
        :class:`~openstack.exceptions.ResourceNotFound` will be \
        raised when the resource does not exist. \
        When set to ``True``, None will be returned when \
        attempting to find a nonexistent resource.

        :returns: One :class:`~otcextensions.sdk.dds.v3.instance.Instance` \
        or None.

        """
        return self._find(_instance.Instance,
                          name_or_id,
                          ignore_missing=ignore_missing)

    def instances(self, **params):
        """Return a generator of instances

        :param dict params: Optional query parameters to be sent to limit \
        the instances being returned.

        :returns: A generator of instance objects.
        :rtype: :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        return self._list(_instance.Instance, **params)

    def get_job(self, job):
        """Get information about a job

        :param job: The value can be either the ID of a job or a \
        :class:`~otcextensions.sdk.dds.v3.job.Job` job.

        :returns: One :class:`~otcextensions.sdk.dds.v3.job.Job`

        """
        return self._get(_job.Job, job)

    def wait_job(self, job, status='Completed', failures=None,
                 interval=20, wait=None):
        """Wait for the job to complete

        :param job: The value can be either the ID of a job or a \
        :param status: Desired status of the job.
        :param failures: List of failure statuses.
        :param interval: Seconds to wait between retries.
        :param wait: Seconds to wait for response.

        :returns: One :class:`~otcextensions.sdk.dds.v3.job.Job`

        """
        if failures is None:
            failures = ['ERROR']
        job = self._find(_job.Job, job, failures=failures)
        if job is not None:
            return resource.wait_for_status(self, job, status, failures,
                                            interval, wait)

    def wait_normal_instance(self, instance, status='normal',
                             failures=None, interval=60, wait=None):
        """Wait for normal status of an instance

        :param instance: The value can be either the ID of an instance or a \
        :class:`~otcextensions.sdk.dds.v3.instance.Instance` instance.

        :param status: The status of the instance to wait for.
        :param failures: The list of failures.
        :param interval: The number of seconds to wait between failures.
        :param wait: Seconds to wait for response.

        :returns: One :class:`~otcextensions.sdk.dds.v3.instance.Instance`

        """
        if failures is None:
            failures = ['ERROR']
        instance = self.get_instance(instance)
        return resource.wait_for_status(self, instance, status,
                                        failures, interval, wait)

    def bind_eip(self, node, public_ip, public_ip_id):
        """Bind an IP to a node

        :param node: The value is the ID of a node
        :param public_ip: The public IP address of the node.
        :param public_ip_id: The public IP address of the node.

        :returns: The IP address of the node.

        """
        eip = _eip.Eip()
        return eip.bind(self, node, public_ip, public_ip_id)

    def unbind_eip(self, node):
        """Unbind an IP to a node

        :param node: The value is the ID of a node

        :returns: The ID of the node.

        """
        eip = _eip.Eip()
        return eip.unbind(self, node)

    def get_policy(self):
        """Get the current policy

        :returns: The current policy.

        """
        return self._get(_recycle_policy.RecyclePolicy, requires_id=False)

    def create_policy(self, **attrs):
        """Create a new policy from attributes

        :param dict attrs: Keyword arguments which will be used to create \
        a :class:`~otcextensions.sdk.dds.v3.recycle_policy.RecyclePolicy`, \
        comprised of the properties on the RecyclePolicy class.

        :returns: The result of creation.

        :rtype: :class:`~otcextensions.sdk.dds.v3.recycle_policy.RecyclePolicy`

        """
        return self._create(_recycle_policy.RecyclePolicy, **attrs)

    def recycle_instances(self, **params):
        """Get list of instances in recycle bin

        :param dict params: Keyword arguments which will be used to get list \
        :returns: A generator of recycle instance objects.

        """
        base_path = _recycle_instance.RecycleInstance.base_path
        return self._list(_recycle_instance.RecycleInstance, **params,
                          base_path=base_path)
